--[[--------------------------------------------------------------------------

  luawinapi - winapi wrapper for Lua
  Copyright (C) 2011 Klaus Oberhofer. See copyright notice in
  LICENSE file

--]]--------------------------------------------------------------------------

require ".\\winapi\\winapi-loader"

local winapi, ERR, TYPE, AUTH = load_luawinapi()

local bit = require("bit")

local bnot = bit.bnot
local band, bor, bxor = bit.band, bit.bor, bit.bxor
local lshift, rshift, rol = bit.lshift, bit.rshift, bit.rol


---------------------------------------------------------------------
-- error constants
S_OK = 0

---------------------------------------------------------------------
-- version constants

VER_PLATFORM_WIN32s            = 0
VER_PLATFORM_WIN32_WINDOWS     = 1
VER_PLATFORM_WIN32_NT          = 2
VER_PLATFORM_WIN32_HH          = 3
VER_PLATFORM_WIN32_CE          = 3

---------------------------------------------------------------------
-- determine and set UNDER_CE, WINVER, _WIN32_WINNT and _WIN32_WINDOWS
do
  local versionInfo = winapi.OSVERSIONINFOW:new()
  versionInfo.dwOSVersionInfoSize = #versionInfo

  local errcode = winapi.GetVersionExW(versionInfo)
  if (0 ~= errcode) then
    UNDER_CE = (VER_PLATFORM_WIN32_CE == versionInfo.dwPlatformId)
    WINVER = bor(lshift(versionInfo.dwMajorVersion, 8), versionInfo.dwMinorVersion)
    if (VER_PLATFORM_WIN32_NT == versionInfo.dwPlatformId) then
      _WIN32_WINNT   = WINVER
    elseif (VER_PLATFORM_WIN32_WINDOWS == versionInfo.dwPlatformId) then
      _WIN32_WINDOWS = WINVER
    end
  else
    error("GetOSVersionW failed: " .. errcode .. " " .. winapi.GetLastError())
  end
end

-- TODO auto ie/comctl detection
-- WIN32_IE = 0x0550

---------------------------------------------------------------------
-- converts ASCII to UCS2
function toUCS2(text)
  local result = {}
  for idx = 1, string.len(text) do
    table.insert(result, string.sub(text, idx, idx) .. "\000")
  end
  return table.concat(result);
end

function toUCS2Z(text)
  local result = {}
  for idx = 1, string.len(text) do
    table.insert(result, string.sub(text, idx, idx) .. "\000")
  end
  table.insert(result, "\000\000")
  return table.concat(result);
end

-- for convenience
_T = toUCS2Z

---------------------------------------------------------------------
-- converts UCS2 to ASCII
function toASCII(text)
  local result = {}
  for idx = 1, string.len(text), 2 do
    table.insert(result, string.sub(text, idx, idx))
  end
  return table.concat(result);
end

---------------------------------------------------------------------
-- converts bool to BOOL
function toBOOL(value)
  if (value) then
    return TRUE
  else
    return FALSE
  end
end

---------------------------------------------------------------------
-- define GUIDs
function DEFINE_GUID(a, b, c, d1, d2, d3, d4, d4, d6, d7, d8)
  local res = winapi.GUID:new()
  res.Data1 = a
  res.Data2 = b
  res.Data3 = c
  res.Data4 = {d1, d2, d3, d4, d4, d6, d7, d8}
  return res
end

---------------------------------------------------------------------
-- kernel32

STARTF_USESHOWWINDOW    = 0x00000001
STARTF_USESIZE          = 0x00000002
STARTF_USEPOSITION      = 0x00000004
STARTF_USECOUNTCHARS    = 0x00000008
STARTF_USEFILLATTRIBUTE = 0x00000010
STARTF_RUNFULLSCREEN    = 0x00000020
STARTF_FORCEONFEEDBACK  = 0x00000040
STARTF_FORCEOFFFEEDBACK = 0x00000080
STARTF_USESTDHANDLES    = 0x00000100

-- #if (WINVER >= 0x0400)
STARTF_USEHOTKEY        = 0x00000200
STARTF_TITLEISLINKNAME  = 0x00000800
STARTF_TITLEISAPPID     = 0x00001000
STARTF_PREVENTPINNING   = 0x00002000
-- #endif /* WINVER >= 0x0400 */

DEBUG_PROCESS = 0x00000001
DEBUG_ONLY_THIS_PROCESS = 0x00000002
CREATE_SUSPENDED = 0x00000004
DETACHED_PROCESS = 0x00000008

CREATE_NEW_CONSOLE = 0x00000010
NORMAL_PRIORITY_CLASS = 0x00000020
IDLE_PRIORITY_CLASS = 0x00000040
HIGH_PRIORITY_CLASS = 0x00000080

REALTIME_PRIORITY_CLASS = 0x00000100
CREATE_NEW_PROCESS_GROUP = 0x00000200
CREATE_UNICODE_ENVIRONMENT = 0x00000400
CREATE_SEPARATE_WOW_VDM = 0x00000800

CREATE_SHARED_WOW_VDM = 0x00001000
CREATE_FORCEDOS = 0x00002000
BELOW_NORMAL_PRIORITY_CLASS = 0x00004000
ABOVE_NORMAL_PRIORITY_CLASS = 0x00008000

INHERIT_PARENT_AFFINITY = 0x00010000
INHERIT_CALLER_PRIORITY = 0x00020000
CREATE_PROTECTED_PROCESS = 0x00040000
EXTENDED_STARTUPINFO_PRESENT = 0x00080000

PROCESS_MODE_BACKGROUND_BEGIN = 0x00100000
PROCESS_MODE_BACKGROUND_END = 0x00200000

CREATE_BREAKAWAY_FROM_JOB = 0x01000000
CREATE_PRESERVE_CODE_AUTHZ_LEVEL = 0x02000000
CREATE_DEFAULT_ERROR_MODE = 0x04000000
CREATE_NO_WINDOW = 0x08000000

PROFILE_USER = 0x10000000
PROFILE_KERNEL = 0x20000000
PROFILE_SERVER = 0x40000000
CREATE_IGNORE_SYSTEM_DEFAULT = 0x80000000

---------------------------------------------------------------------
-- notification messages

-- Generic WM_NOTIFY notifications
NM_FIRST = ( 0xffffffff + 1 )

NM_OUTOFMEMORY = (NM_FIRST-1)
NM_CLICK = (NM_FIRST-2)
NM_DBLCLK = (NM_FIRST-3)
NM_RETURN = (NM_FIRST-4)
NM_RCLICK = (NM_FIRST-5)
NM_RDBLCLK = (NM_FIRST-6)
NM_SETFOCUS = (NM_FIRST-7)
NM_KILLFOCUS = (NM_FIRST-8)
NM_CUSTOMDRAW = (NM_FIRST-12)
NM_HOVER = (NM_FIRST-13)
-- #if (_WIN32_IE >= 0x0400)
NM_NCHITTEST = (NM_FIRST-14)
NM_KEYDOWN = (NM_FIRST-15)
NM_RELEASEDCAPTURE = (NM_FIRST-16)
NM_SETCURSOR = (NM_FIRST-17)
NM_CHAR = (NM_FIRST-18)
-- #endif
-- #if (_WIN32_IE >= 0x0401)
NM_TOOLTIPSCREATED = (NM_FIRST-19)
-- #endif
-- #if (_WIN32_IE >= 0x0500)
NM_LDOWN = (NM_FIRST-20)
NM_RDOWN = (NM_FIRST-21)
NM_THEMECHANGED = (NM_FIRST-22)
-- #endif
-- #if (_WIN32_WINNT >= 0x0600)
NM_FONTCHANGED = (NM_FIRST-23)
NM_CUSTOMTEXT = (NM_FIRST-24)
NM_TVSTATEIMAGECHANGING = (NM_FIRST-24)
-- #endif

---------------------------------------------------------------------
-- error constants
S_OK = 0

-- messages
MSG_CONSTANTS = {
    [0x0000] =  "WM_NULL";
    [0x0001] =  "WM_CREATE";
    [0x0002] =  "WM_DESTROY";
    [0x0003] =  "WM_MOVE";
    [0x0005] =  "WM_SIZE";
    [0x0006] =  "WM_ACTIVATE";
    [0x0007] =  "WM_SETFOCUS";
    [0x0008] =  "WM_KILLFOCUS";
    [0x000A] =  "WM_ENABLE";
    [0x000B] =  "WM_SETREDRAW";
    [0x000C] =  "WM_SETTEXT";
    [0x000D] =  "WM_GETTEXT";
    [0x000E] =  "WM_GETTEXTLENGTH";
    [0x000F] =  "WM_PAINT";
    [0x0010] =  "WM_CLOSE";

    [0x0011] =  "WM_QUERYENDSESSION";
    [0x0013] =  "WM_QUERYOPEN";
    [0x0016] =  "WM_ENDSESSION";
    [0x0012] =  "WM_QUIT";
    [0x0014] =  "WM_ERASEBKGND";
    [0x0015] =  "WM_SYSCOLORCHANGE";
    [0x0018] =  "WM_SHOWWINDOW";
    [0x001A] =  "WM_WININICHANGE";
    ["WM_WININICHANGE"] =  "WM_SETTINGCHANGE";
    [0x001B] =  "WM_DEVMODECHANGE";
    [0x001C] =  "WM_ACTIVATEAPP";
    [0x001D] =  "WM_FONTCHANGE";
    [0x001E] =  "WM_TIMECHANGE";
    [0x001F] =  "WM_CANCELMODE";
    [0x0020] =  "WM_SETCURSOR";
    [0x0021] =  "WM_MOUSEACTIVATE";
    [0x0022] =  "WM_CHILDACTIVATE";
    [0x0023] =  "WM_QUEUESYNC";
    [0x0024] =  "WM_GETMINMAXINFO";
    [0x0026] =  "WM_PAINTICON";
    [0x0027] =  "WM_ICONERASEBKGND";
    [0x0028] =  "WM_NEXTDLGCTL";
    [0x002A] =  "WM_SPOOLERSTATUS";
    [0x002B] =  "WM_DRAWITEM";
    [0x002C] =  "WM_MEASUREITEM";
    [0x002D] =  "WM_DELETEITEM";
    [0x002E] =  "WM_VKEYTOITEM";
    [0x002F] =  "WM_CHARTOITEM";
    [0x0030] =  "WM_SETFONT";
    [0x0031] =  "WM_GETFONT";
    [0x0032] =  "WM_SETHOTKEY";
    [0x0033] =  "WM_GETHOTKEY";
    [0x0037] =  "WM_QUERYDRAGICON";
    [0x0039] =  "WM_COMPAREITEM";

    [0x003D] =  "WM_GETOBJECT";
    [0x0041] =  "WM_COMPACTING";
    [0x0044] =  "WM_COMMNOTIFY";
    [0x0046] =  "WM_WINDOWPOSCHANGING";
    [0x0047] =  "WM_WINDOWPOSCHANGED";
    [0x0048] =  "WM_POWER";
    [0x004A] =  "WM_COPYDATA";
    [0x004B] =  "WM_CANCELJOURNAL";
    [0x004E] =  "WM_NOTIFY";
    [0x0050] =  "WM_INPUTLANGCHANGEREQUEST";
    [0x0051] =  "WM_INPUTLANGCHANGE";
    [0x0052] =  "WM_TCARD";
    [0x0053] =  "WM_HELP";
    [0x0054] =  "WM_USERCHANGED";
    [0x0055] =  "WM_NOTIFYFORMAT";
    [0x007B] =  "WM_CONTEXTMENU";
    [0x007C] =  "WM_STYLECHANGING";
    [0x007D] =  "WM_STYLECHANGED";
    [0x007E] =  "WM_DISPLAYCHANGE";
    [0x007F] =  "WM_GETICON";
    [0x0080] =  "WM_SETICON";

    [0x0081] =  "WM_NCCREATE";
    [0x0082] =  "WM_NCDESTROY";
    [0x0083] =  "WM_NCCALCSIZE";
    [0x0084] =  "WM_NCHITTEST";
    [0x0085] =  "WM_NCPAINT";
    [0x0086] =  "WM_NCACTIVATE";
    [0x0087] =  "WM_GETDLGCODE";

    [0x0088] =  "WM_SYNCPAINT";

    [0x00A0] =  "WM_NCMOUSEMOVE";
    [0x00A1] =  "WM_NCLBUTTONDOWN";
    [0x00A2] =  "WM_NCLBUTTONUP";
    [0x00A3] =  "WM_NCLBUTTONDBLCLK";
    [0x00A4] =  "WM_NCRBUTTONDOWN";
    [0x00A5] =  "WM_NCRBUTTONUP";
    [0x00A6] =  "WM_NCRBUTTONDBLCLK";
    [0x00A7] =  "WM_NCMBUTTONDOWN";
    [0x00A8] =  "WM_NCMBUTTONUP";
    [0x00A9] =  "WM_NCMBUTTONDBLCLK";
    [0x00AB] =  "WM_NCXBUTTONDOWN";
    [0x00AC] =  "WM_NCXBUTTONUP";
    [0x00AD] =  "WM_NCXBUTTONDBLCLK";

--    [0x0100] = "WM_KEYFIRST"
    [0x0100] = "WM_KEYDOWN";
    [0x0101] = "WM_KEYUP";
    [0x0102] = "WM_CHAR";
    [0x0103] = "WM_DEADCHAR";
    [0x0104] = "WM_SYSKEYDOWN";
    [0x0105] = "WM_SYSKEYUP";
    [0x0106] = "WM_SYSCHAR";
    [0x0107] = "WM_SYSDEADCHAR";
--    [0x0108 WM_KEYLAST

    [0x010D] = "WM_IME_STARTCOMPOSITION",
    [0x010E] = "WM_IME_ENDCOMPOSITION",
    [0x010F] = "WM_IME_COMPOSITION",
    [0x010F] = "WM_IME_KEYLAST",
    [0x0110] = "WM_INITDIALOG",
    [0x0111] = "WM_COMMAND",
    [0x0112] = "WM_SYSCOMMAND",
    [0x0113] = "WM_TIMER",
    [0x0114] = "WM_HSCROLL",
    [0x0115] = "WM_VSCROLL",
    [0x0116] = "WM_INITMENU",
    [0x0117] = "WM_INITMENUPOPUP",
    [0x0119] = "WM_GESTURE",
    [0x011A] = "WM_GESTURENOTIFY",
    [0x011F] = "WM_MENUSELECT",
    [0x0120] = "WM_MENUCHAR",
    [0x0121] = "WM_ENTERIDLE",

    [0x0122] = "WM_MENURBUTTONUP",
    [0x0123] = "WM_MENUDRAG",
    [0x0124] = "WM_MENUGETOBJECT",
    [0x0125] = "WM_UNINITMENUPOPUP",
    [0x0126] = "WM_MENUCOMMAND",
    [0x0127] = "WM_CHANGEUISTATE",
    [0x0128] = "WM_UPDATEUISTATE",
    [0x0129] = "WM_QUERYUISTATE",

    [0x0132] = "WM_CTLCOLORMSGBOX",
    [0x0133] = "WM_CTLCOLOREDIT",
    [0x0134] = "WM_CTLCOLORLISTBOX",
    [0x0135] = "WM_CTLCOLORBTN",
    [0x0136] = "WM_CTLCOLORDLG",
    [0x0137] = "WM_CTLCOLORSCROLLBAR",
    [0x0138] = "WM_CTLCOLORSTATIC",


--    [0x0200] =  "WM_MOUSEFIRST  0x0200
    [0x0200] =  "WM_MOUSEMOVE",
    [0x0201] =  "WM_LBUTTONDOWN",
    [0x0202] =  "WM_LBUTTONUP",
    [0x0203] =  "WM_LBUTTONDBLCLK",
    [0x0204] =  "WM_RBUTTONDOWN",
    [0x0205] =  "WM_RBUTTONUP",
    [0x0206] =  "WM_RBUTTONDBLCLK",
    [0x0207] =  "WM_MBUTTONDOWN",
    [0x0208] =  "WM_MBUTTONUP",
    [0x0209] =  "WM_MBUTTONDBLCLK",


    [0x0210] =  "WM_PARENTNOTIFY";
    [0x0211] =  "WM_ENTERMENULOOP";
    [0x0212] =  "WM_EXITMENULOOP";

    [0x0213] =  "WM_NEXTMENU";
    [0x0214] =  "WM_SIZING";
    [0x0215] =  "WM_CAPTURECHANGED";
    [0x0216] =  "WM_MOVING";

    [0x0218] =  "WM_POWERBROADCAST";

    [0x0219] =  "WM_DEVICECHANGE";

    [0x0281] =  "WM_IME_SETCONTEXT";
    [0x0282] =  "WM_IME_NOTIFY";
    [0x0283] =  "WM_IME_CONTROL";
    [0x0284] =  "WM_IME_COMPOSITIONFULL";
    [0x0285] =  "WM_IME_SELECT";
    [0x0286] =  "WM_IME_CHAR";

    [0x0312] =  "WM_HOTKEY";

    [0x0400] =  "WM_USER";

    [0x8000] =  "WM_APP";
}

if (UNDER_CE) then
  MSG_CONSTANTS[0x03FD] = "WM_DBNOTIFICATION"
  MSG_CONSTANTS[0x03FE] = "WM_NETCONNECT"
  MSG_CONSTANTS[0x03FF] = "WM_HIBERNATE"
end

-- map messages to global constants
for k,v in pairs(MSG_CONSTANTS) do
    _G[v] = k
end


-- window styles
WS_OVERLAPPED = 0x00000000
WS_POPUP = 0x80000000
WS_CHILD = 0x40000000
WS_MINIMIZE = 0x20000000
WS_VISIBLE = 0x10000000
WS_DISABLED = 0x08000000
WS_CLIPSIBLINGS = 0x04000000
WS_CLIPCHILDREN = 0x02000000
WS_MAXIMIZE = 0x01000000
WS_CAPTION = 0x00C00000
WS_BORDER = 0x00800000
WS_DLGFRAME = 0x00400000
WS_VSCROLL = 0x00200000
WS_HSCROLL = 0x00100000
WS_SYSMENU = 0x00080000
WS_THICKFRAME = 0x00040000
WS_GROUP = 0x00020000
WS_TABSTOP = 0x00010000

if (UNDER_CE) then
  WS_MAXIMIZEBOX = 0x00020000
  WS_MINIMIZEBOX = 0x00010000

  -- PocketPC specific
  WS_NONAVDONEBUTTON = WS_MINIMIZEBOX
else
  WS_MINIMIZEBOX = 0x00020000
  WS_MAXIMIZEBOX = 0x00010000
end

WS_TILED = WS_OVERLAPPED
WS_ICONIC = WS_MINIMIZE
WS_SIZEBOX = WS_THICKFRAME
WS_OVERLAPPEDWINDOW = bor(WS_OVERLAPPED, WS_CAPTION, WS_SYSMENU, WS_THICKFRAME, WS_MINIMIZEBOX, WS_MAXIMIZEBOX)
WS_TILEDWINDOW = WS_OVERLAPPEDWINDOW
WS_POPUPWINDOW = bor(WS_POPUP, WS_BORDER, WS_SYSMENU)
WS_CHILDWINDOW = WS_CHILD

-- window extended styles
WS_EX_DLGMODALFRAME = 0x00000001
WS_EX_NOPARENTNOTIFY = 0x00000004
WS_EX_TOPMOST = 0x00000008
WS_EX_ACCEPTFILES = 0x00000010
WS_EX_TRANSPARENT = 0x00000020
WS_EX_MDICHILD = 0x00000040
WS_EX_TOOLWINDOW = 0x00000080
WS_EX_WINDOWEDGE = 0x00000100
WS_EX_CLIENTEDGE = 0x00000200
WS_EX_CONTEXTHELP = 0x00000400
WS_EX_RIGHT = 0x00001000
WS_EX_LEFT = 0x00000000
WS_EX_RTLREADING = 0x00002000
WS_EX_LTRREADING = 0x00000000
WS_EX_LEFTSCROLLBAR = 0x00004000
WS_EX_RIGHTSCROLLBAR = 0x00000000
WS_EX_CONTROLPARENT = 0x00010000
WS_EX_STATICEDGE = 0x00020000
WS_EX_APPWINDOW = 0x00040000
WS_EX_NOINHERITLAYOUT = 0x00100000
WS_EX_LAYOUTRTL = 0x00400000
WS_EX_OVERLAPPEDWINDOW = bor(WS_EX_WINDOWEDGE, WS_EX_CLIENTEDGE)
WS_EX_PALETTEWINDOW = bor(WS_EX_WINDOWEDGE,WS_EX_TOOLWINDOW,WS_EX_TOPMOST)

WS_EX_CAPTIONOKBTN = 0x80000000
WS_EX_NODRAG = 0x40000000
WS_EX_ABOVESTARTUP = 0x20000000
WS_EX_INK = 0x10000000
WS_EX_NOANIMATION = 0x04000000


PM_NOREMOVE = 0x0000
PM_REMOVE = 0x0001
PM_NOYIELD = 0x0002

-- window class/creation
CW_USEDEFAULT = 0x80000000

-- WM_MOUSEACTIVATE
MA_ACTIVATE = 1
MA_ACTIVATEANDEAT = 2
MA_NOACTIVATE = 3
MA_NOACTIVATEANDEAT = 4

--
GWL_WNDPROC     =  -4
GWL_HINSTANCE   =  -6
GWL_HWNDPARENT  =  -8
GWL_ID          = -12
GWL_STYLE       = -16
GWL_EXSTYLE     = -20
GWL_USERDATA    = -21

CS_HREDRAW     = 2
CS_VREDRAW     = 1
CS_DBLCLKS = 0x0008
CS_OWNDC = 0x0020
CS_CLASSDC = 0x0040
CS_PARENTDC = 0x0080
CS_NOCLOSE = 0x0200
CS_SAVEBITS = 0x0800
CS_BYTEALIGNCLIENT = 0x1000
CS_BYTEALIGNWINDOW = 0x2000
CS_GLOBALCLASS = 0x4000
CS_IME = 0x00010000
CS_DROPSHADOW = 0x00020000


DFC_CAPTION = 1
DFC_MENU = 2
DFC_SCROLL = 3
DFC_BUTTON = 4
DFC_POPUPMENU = 5

PRF_CHECKVISIBLE = 0x00000001
PRF_NONCLIENT = 0x00000002
PRF_CLIENT = 0x00000004
PRF_ERASEBKGND = 0x00000008
PRF_CHILDREN = 0x00000010
PRF_OWNED = 0x00000020

BDR_RAISEDOUTER = 0x0001
BDR_SUNKENOUTER = 0x0002
BDR_RAISEDINNER = 0x0004
BDR_SUNKENINNER = 0x0008

BDR_OUTER = bor(BDR_RAISEDOUTER, BDR_SUNKENOUTER)
BDR_INNER = bor(BDR_RAISEDINNER, BDR_SUNKENINNER)
BDR_RAISED = bor(BDR_RAISEDOUTER, BDR_RAISEDINNER)
BDR_SUNKEN = bor(BDR_SUNKENOUTER, BDR_SUNKENINNER)

EDGE_RAISED = bor(BDR_RAISEDOUTER, BDR_RAISEDINNER)
EDGE_SUNKEN = bor(BDR_SUNKENOUTER, BDR_SUNKENINNER)
EDGE_ETCHED = bor(BDR_SUNKENOUTER, BDR_RAISEDINNER)
EDGE_BUMP = bor(BDR_RAISEDOUTER, BDR_SUNKENINNER)

BF_LEFT = 0x0001
BF_TOP = 0x0002
BF_RIGHT = 0x0004
BF_BOTTOM = 0x0008

BF_TOPLEFT = bor(BF_TOP, BF_LEFT)
BF_TOPRIGHT = bor(BF_TOP, BF_RIGHT)
BF_BOTTOMLEFT = bor(BF_BOTTOM, BF_LEFT)
BF_BOTTOMRIGHT = bor(BF_BOTTOM, BF_RIGHT)
BF_RECT = bor(BF_LEFT, BF_TOP, BF_RIGHT, BF_BOTTOM)

BF_DIAGONAL = 0x0010

BF_DIAGONAL_ENDTOPRIGHT = bor(BF_DIAGONAL, BF_TOP, BF_RIGHT)
BF_DIAGONAL_ENDTOPLEFT = bor(BF_DIAGONAL, BF_TOP, BF_LEFT)
BF_DIAGONAL_ENDBOTTOMLEFT = bor(BF_DIAGONAL, BF_BOTTOM, BF_LEFT)
BF_DIAGONAL_ENDBOTTOMRIGHT = bor(BF_DIAGONAL, BF_BOTTOM, BF_RIGHT)

BF_MIDDLE = 0x0800
BF_SOFT = 0x1000
BF_ADJUST = 0x2000
BF_FLAT = 0x4000
BF_MONO = 0x8000

DFC_CAPTION = 1
DFC_MENU = 2
DFC_SCROLL = 3
DFC_BUTTON = 4
DFC_POPUPMENU = 5

DFCS_CAPTIONCLOSE = 0x0000
DFCS_CAPTIONMIN = 0x0001
DFCS_CAPTIONMAX = 0x0002
DFCS_CAPTIONRESTORE = 0x0003
DFCS_CAPTIONHELP = 0x0004
DFCS_MENUARROW = 0x0000
DFCS_MENUCHECK = 0x0001
DFCS_MENUBULLET = 0x0002
DFCS_MENUARROWRIGHT = 0x0004
DFCS_SCROLLUP = 0x0000
DFCS_SCROLLDOWN = 0x0001
DFCS_SCROLLLEFT = 0x0002
DFCS_SCROLLRIGHT = 0x0003
DFCS_SCROLLCOMBOBOX = 0x0005
DFCS_SCROLLSIZEGRIP = 0x0008
DFCS_SCROLLSIZEGRIPRIGHT = 0x0010
DFCS_BUTTONCHECK = 0x0000
DFCS_BUTTONRADIOIMAGE = 0x0001
DFCS_BUTTONRADIOMASK = 0x0002
DFCS_BUTTONRADIO = 0x0004
DFCS_BUTTON3STATE = 0x0008
DFCS_BUTTONPUSH = 0x0010
DFCS_INACTIVE = 0x0100
DFCS_PUSHED = 0x0200
DFCS_CHECKED = 0x0400
DFCS_TRANSPARENT = 0x0800
DFCS_HOT = 0x1000
DFCS_ADJUSTRECT = 0x2000
DFCS_FLAT = 0x4000
DFCS_MONO = 0x8000

-- window show/hide
SW_HIDE = 0
SW_SHOWNORMAL = 1
SW_NORMAL = 1
SW_SHOWMINIMIZED = 2
SW_SHOWMAXIMIZED = 3
SW_MAXIMIZE = 3
SW_SHOWNOACTIVATE = 4
SW_SHOW = 5
SW_MINIMIZE = 6
SW_SHOWMINNOACTIVE = 7
SW_SHOWNA = 8
SW_RESTORE = 9
SW_SHOWDEFAULT = 10
SW_FORCEMINIMIZE = 11
SW_MAX = 11

HIDE_WINDOW = 0
SHOW_OPENWINDOW = 1
SHOW_ICONWINDOW = 2
SHOW_FULLSCREEN = 3
SHOW_OPENNOACTIVATE = 4

SW_PARENTCLOSING = 1
SW_OTHERZOOM = 2
SW_PARENTOPENING = 3
SW_OTHERUNZOOM = 4


IDI_APPLICATION = 32512

BS_PUSHBUTTON = 0x00000000
BS_DEFPUSHBUTTON = 0x00000001
BS_CHECKBOX = 0x00000002
BS_AUTOCHECKBOX = 0x00000003
BS_RADIOBUTTON = 0x00000004
BS_3STATE = 0x00000005
BS_AUTO3STATE = 0x00000006
BS_GROUPBOX = 0x00000007
BS_AUTORADIOBUTTON = 0x00000009
BS_OWNERDRAW = 0x0000000B
BS_LEFTTEXT = 0x00000020
--#if (WINVER >= 0x0400)
BS_TEXT = 0x00000000
BS_LEFT = 0x00000100
BS_RIGHT = 0x00000200
BS_CENTER = 0x00000300
BS_TOP = 0x00000400
BS_BOTTOM = 0x00000800
BS_VCENTER = 0x00000C00
BS_PUSHLIKE = 0x00001000
BS_MULTILINE = 0x00002000
BS_NOTIFY = 0x00004000
BS_RIGHTBUTTON = BS_LEFTTEXT
--#endif /* WINVER >= 0x0400 */

BN_CLICKED     = 0
BN_PAINT       = 1
BN_DBLCLK      = 5
BN_SETFOCUS    = 6
BN_KILLFOCUS   = 7

MOD_ALT = 0x0001
MOD_CONTROL = 0x0002
MOD_SHIFT = 0x0004
MOD_WIN = 0x0008
MOD_KEYUP = 0x1000

VK_LBUTTON = 0x01
VK_RBUTTON = 0x02
VK_CANCEL = 0x03
VK_MBUTTON = 0x04
VK_XBUTTON1 = 0x05
VK_XBUTTON2 = 0x06

VK_BACK = 0x08
VK_TAB = 0x09

VK_CLEAR = 0x0C
VK_RETURN = 0x0D

VK_SHIFT = 0x10
VK_CONTROL = 0x11
VK_MENU = 0x12
VK_PAUSE = 0x13
VK_CAPITAL = 0x14

VK_KANA = 0x15
VK_HANGEUL = 0x15
VK_HANGUL = 0x15
VK_JUNJA = 0x17
VK_FINAL = 0x18
VK_HANJA = 0x19
VK_KANJI = 0x19

VK_ESCAPE = 0x1B

VK_CONVERT = 0x1c
VK_NOCONVERT = 0x1d

VK_SPACE = 0x20
VK_PRIOR = 0x21
VK_NEXT = 0x22
VK_END = 0x23
VK_HOME = 0x24
VK_LEFT = 0x25
VK_UP = 0x26
VK_RIGHT = 0x27
VK_DOWN = 0x28
VK_SELECT = 0x29
VK_PRINT = 0x2A
VK_EXECUTE = 0x2B
VK_SNAPSHOT = 0x2C
VK_INSERT = 0x2D
VK_DELETE = 0x2E
VK_HELP = 0x2F

VK_LWIN = 0x5B
VK_RWIN = 0x5C
VK_APPS = 0x5D

VK_SLEEP = 0x5F

VK_NUMPAD0 = 0x60
VK_NUMPAD1 = 0x61
VK_NUMPAD2 = 0x62
VK_NUMPAD3 = 0x63
VK_NUMPAD4 = 0x64
VK_NUMPAD5 = 0x65
VK_NUMPAD6 = 0x66
VK_NUMPAD7 = 0x67
VK_NUMPAD8 = 0x68
VK_NUMPAD9 = 0x69
VK_MULTIPLY = 0x6A
VK_ADD = 0x6B
VK_SEPARATOR = 0x6C
VK_SUBTRACT = 0x6D
VK_DECIMAL = 0x6E
VK_DIVIDE = 0x6F
VK_F1 = 0x70
VK_F2 = 0x71
VK_F3 = 0x72
VK_F4 = 0x73
VK_F5 = 0x74
VK_F6 = 0x75
VK_F7 = 0x76
VK_F8 = 0x77
VK_F9 = 0x78
VK_F10 = 0x79
VK_F11 = 0x7A
VK_F12 = 0x7B
VK_F13 = 0x7C
VK_F14 = 0x7D
VK_F15 = 0x7E
VK_F16 = 0x7F
VK_F17 = 0x80
VK_F18 = 0x81
VK_F19 = 0x82
VK_F20 = 0x83
VK_F21 = 0x84
VK_F22 = 0x85
VK_F23 = 0x86
VK_F24 = 0x87

VK_NUMLOCK = 0x90
VK_SCROLL = 0x91

VK_LSHIFT = 0xA0
VK_RSHIFT = 0xA1
VK_LCONTROL = 0xA2
VK_RCONTROL = 0xA3
VK_LMENU = 0xA4
VK_RMENU = 0xA5

VK_EXTEND_BSLASH = 0xE2
VK_OEM_102 = 0xE2

VK_PROCESSKEY = 0xE5

VK_ATTN = 0xF6
VK_CRSEL = 0xF7
VK_EXSEL = 0xF8
VK_EREOF = 0xF9
VK_PLAY = 0xFA
VK_ZOOM = 0xFB
VK_NONAME = 0xFC
VK_PA1 = 0xFD
VK_OEM_CLEAR = 0xFE


VK_SEMICOLON = 0xBA
VK_EQUAL = 0xBB
VK_COMMA = 0xBC
VK_HYPHEN = 0xBD
VK_PERIOD = 0xBE
VK_SLASH = 0xBF
VK_BACKQUOTE = 0xC0

VK_BROWSER_BACK = 0xA6
VK_BROWSER_FORWARD = 0xA7
VK_BROWSER_REFRESH = 0xA8
VK_BROWSER_STOP = 0xA9
VK_BROWSER_SEARCH = 0xAA
VK_BROWSER_FAVORITES = 0xAB
VK_BROWSER_HOME = 0xAC
VK_VOLUME_MUTE = 0xAD
VK_VOLUME_DOWN = 0xAE
VK_VOLUME_UP = 0xAF
VK_MEDIA_NEXT_TRACK = 0xB0
VK_MEDIA_PREV_TRACK = 0xB1
VK_MEDIA_STOP = 0xB2
VK_MEDIA_PLAY_PAUSE = 0xB3
VK_LAUNCH_MAIL = 0xB4
VK_LAUNCH_MEDIA_SELECT = 0xB5
VK_LAUNCH_APP1 = 0xB6
VK_LAUNCH_APP2 = 0xB7

VK_LBRACKET = 0xDB
VK_BACKSLASH = 0xDC
VK_RBRACKET = 0xDD
VK_APOSTROPHE = 0xDE
VK_OFF = 0xDF

VK_DBE_ALPHANUMERIC = 0x0f0
VK_DBE_KATAKANA = 0x0f1
VK_DBE_HIRAGANA = 0x0f2
VK_DBE_SBCSCHAR = 0x0f3
VK_DBE_DBCSCHAR = 0x0f4
VK_DBE_ROMAN = 0x0f5
VK_DBE_NOROMAN = 0x0f6
VK_DBE_ENTERWORDREGISTERMODE = 0x0f7
VK_DBE_ENTERIMECONFIGMODE = 0x0f8
VK_DBE_FLUSHSTRING = 0x0f9
VK_DBE_CODEINPUT = 0x0fa
VK_DBE_NOCODEINPUT = 0x0fb
VK_DBE_DETERMINESTRING = 0x0fc
VK_DBE_ENTERDLGCONVERSIONMODE = 0x0fd

if (UNDER_CE) then
  VK_TSOFT1 = VK_F1
  VK_TSOFT2 = VK_F2
  VK_TTALK = VK_F3
  VK_TEND = VK_F4
  VK_THOME = VK_LWIN
  VK_TBACK = VK_ESCAPE

  VK_TACTION = VK_RETURN
  VK_TRECORD = VK_F10
  VK_TFLIP = VK_F17
  VK_TPOWER = VK_F18
  VK_TVOLUMEUP = VK_F6
  VK_TVOLUMEDOWN = VK_F7

  VK_TSPEAKERPHONE_TOGGLE = VK_F16
  VK_END_ALL_DATA_CALLS = VK_F15

  VK_TUP = VK_UP
  VK_TDOWN = VK_DOWN
  VK_TLEFT = VK_LEFT
  VK_TRIGHT = VK_RIGHT

--[[
  VK_T0 = L'0'
  VK_T1 = L'1'
  VK_T2 = L'2'
  VK_T3 = L'3'
  VK_T4 = L'4'
  VK_T5 = L'5'
  VK_T6 = L'6'
  VK_T7 = L'7'
  VK_T8 = L'8'
  VK_T9 = L'9'
--]]
  VK_TSTAR = VK_F8
  VK_TPOUND = VK_F9

  VK_SYMBOL = VK_F11

  VK_REDKEY = VK_F19
  VK_ROCKER = VK_F20
  VK_DPAD = VK_F21
  VK_ACTION = VK_F23

  VK_VOICEDIAL = VK_F24
  VK_KEYLOCK = VK_F22

  VK_DONE = VK_F6
  MOD_DONE = bor(MOD_WIN, MOD_KEYUP)

  VK_MOJI = VK_F7
  MOD_MOJI = bor(MOD_WIN, MOD_KEYUP)

  VK_APP_FIRST = 0xC1
  VK_APP1 = 0xC1
  VK_APP2 = 0xC2
  VK_APP3 = 0xC3
  VK_APP4 = 0xC4
  VK_APP5 = 0xC5
  VK_APP6 = 0xC6
  VK_APP_LAST = 0xC6
end

CS_HREDRAW     = 2
CS_VREDRAW     = 1

MIIM_STATE    = 1
MIIM_ID      = 2
MIIM_SUBMENU   = 4
MIIM_CHECKMARKS  = 8
MIIM_TYPE    = 16
MIIM_DATA    = 32
MIIM_STRING    = 64
MIIM_BITMAP    = 128
MIIM_FTYPE     = 256

MFT_BITMAP    = 4
MFT_MENUBARBREAK =32
MFT_MENUBREAK  = 64
MFT_OWNERDRAW  = 256
MFT_RADIOCHECK  = 512
MFT_RIGHTJUSTIFY= 0x4000
MFT_SEPARATOR   = 0x800
MFT_RIGHTORDER  = 0x2000
MFT_STRING     = 0

MF_ENABLED      = 0
MF_GRAYED       = 1
MF_DISABLED     = 2
MF_BITMAP       = 4
MF_CHECKED      = 8
MF_MENUBARBREAK  = 32
MF_MENUBREAK    = 64
MF_OWNERDRAW    = 256
MF_POPUP        = 16
MF_SEPARATOR    = 0x800
MF_STRING       = 0
MF_UNCHECKED    = 0
MF_DEFAULT      = 4096
MF_SYSMENU      = 0x2000
MF_HELP         = 0x4000
MF_END          = 128
MF_RIGHTJUSTIFY  = 0x4000
MF_MOUSESELECT   = 0x8000
MF_INSERT    = 0
MF_CHANGE    = 128
MF_APPEND    = 256
MF_DELETE    = 512
MF_REMOVE    = 4096
MF_USECHECKBITMAPS= 512
MF_UNHILITE    = 0
MF_HILITE    = 128
MF_BYCOMMAND  = 0
MF_BYPOSITION  = 1024
MF_UNCHECKED  = 0
MF_HILITE     = 128
MF_UNHILITE   =  0

LOCALE_SYSTEM_DEFAULT =  0x800

MFS_GRAYED        =  0x00000003
MFS_DISABLED      =  MFS_GRAYED
MFS_CHECKED       =  MF_CHECKED
MFS_HILITE        =  MF_HILITE
MFS_ENABLED       =  MF_ENABLED
MFS_UNCHECKED     =  MF_UNCHECKED
MFS_UNHILITE      =  MF_UNHILITE
MFS_DEFAULT       =  MF_DEFAULT


WA_INACTIVE = 0
WA_ACTIVE = 1
WA_CLICKACTIVE = 2

RB_SETBARINFO = WM_USER + 4
RB_GETBANDCOUNT = WM_USER +  12
RB_INSERTBANDA = WM_USER + 1
RB_INSERTBANDW = WM_USER + 10

RB_INSERTBAND = RB_INSERTBANDA

RBBIM_STYLE = 1
RBBIM_COLORS = 2
RBBIM_TEXT = 4
RBBIM_IMAGE = 8
RBBIM_CHILD = 16
RBBIM_CHILDSIZE = 32
RBBIM_SIZE = 64
RBBIM_BACKGROUND = 128
RBBIM_ID = 256
RBBIM_IDEALSIZE = 0x00000200

TPM_CENTERALIGN =4
TPM_LEFTALIGN =0
TPM_RIGHTALIGN= 8
TPM_LEFTBUTTON= 0
TPM_RIGHTBUTTON= 2
TPM_HORIZONTAL= 0
TPM_VERTICAL= 64
TPM_TOPALIGN= 0
TPM_VCENTERALIGN= 16
TPM_BOTTOMALIGN= 32
TPM_NONOTIFY= 128
TPM_RETURNCMD= 256

TBIF_TEXT = 0x00000002

DT_NOPREFIX   =      0x00000800
DT_HIDEPREFIX =      1048576

WH_CBT       =  5
WH_MSGFILTER =  (-1)

I_IMAGENONE = -2
TBSTATE_ENABLED = 4

BTNS_SHOWTEXT = 0x00000040
CW_USEDEFAULT = 0x80000000

BF_LEFT      = 1
BF_TOP       = 2
BF_RIGHT     = 4
BF_BOTTOM    = 8

BDR_RAISEDOUTER =      1
BDR_SUNKENOUTER =      2
BDR_RAISEDINNER =      4
BDR_SUNKENINNER =      8
BDR_OUTER    = 3
BDR_INNER    = 0xc
BDR_RAISED   = 5
BDR_SUNKEN   = 10

EDGE_RAISED  = bor(BDR_RAISEDOUTER, BDR_RAISEDINNER)
EDGE_SUNKEN  = bor(BDR_SUNKENOUTER, BDR_SUNKENINNER)
EDGE_ETCHED  = bor(BDR_SUNKENOUTER, BDR_RAISEDINNER)
EDGE_BUMP    = bor(BDR_RAISEDOUTER, BDR_SUNKENINNER)

IDC_SIZENWSE   = 32642
IDC_SIZENESW   = 32643
IDC_SIZEWE      = 32644
IDC_SIZENS     = 32645
IDC_SIZEALL   = 32646
IDC_SIZE     = 32640
IDC_ARROW     = 32512

TCIF_TEXT      = 1
TCIF_IMAGE     = 2
TCIF_RTLREADING  = 4
TCIF_PARAM    = 8


TCS_MULTILINE   = 512

MK_LBUTTON      = 1
MK_RBUTTON      = 2
MK_SHIFT        = 4
MK_CONTROL      = 8
MK_MBUTTON      = 16

ILC_COLOR     = 0
ILC_COLOR4     = 4
ILC_COLOR8     = 8
ILC_COLOR16   = 16
ILC_COLOR24   = 24
ILC_COLOR32   = 32
ILC_COLORDDB   = 254
ILC_MASK     = 1
ILC_PALETTE   = 2048

IMAGE_BITMAP   = 0
IMAGE_ICON     = 1

LR_DEFAULTCOLOR = 0x0000
LR_MONOCHROME = 0x0001
LR_COLOR = 0x0002
LR_COPYRETURNORG = 0x0004
LR_COPYDELETEORG = 0x0008
LR_LOADFROMFILE = 0x0010
LR_LOADTRANSPARENT = 0x0020
LR_DEFAULTSIZE = 0x0040
LR_VGACOLOR = 0x0080
LR_LOADMAP3DCOLORS = 0x1000
LR_CREATEDIBSECTION = 0x2000
LR_COPYFROMRESOURCE = 0x4000
LR_SHARED = 0x8000


LVSIL_NORMAL   = 0
LVSIL_SMALL    = 1
LVSIL_STATE    = 2

SRCCOPY     = 0xCC0020

GWL_WNDPROC   = -4

HWND_BOTTOM   = 1
HWND_TOP    = 0
HWND_TOPMOST  =-1

SWP_DRAWFRAME  = 32
SWP_FRAMECHANGED= 32
SWP_HIDEWINDOW  = 128
SWP_NOACTIVATE  = 16
SWP_NOCOPYBITS  = 256
SWP_NOMOVE    = 2
SWP_NOSIZE    = 1
SWP_NOREDRAW  = 8
SWP_NOZORDER  = 4
SWP_SHOWWINDOW  = 64
SWP_NOOWNERZORDER =512
SWP_NOREPOSITION= 512
SWP_NOSENDCHANGING= 1024
SWP_DEFERERASE  = 8192
SWP_ASYNCWINDOWPOS=  16384

DCX_WINDOW     = 1
DCX_CACHE     = 2
DCX_PARENTCLIP   = 32
DCX_CLIPSIBLINGS= 16
DCX_CLIPCHILDREN= 8
DCX_NORESETATTRS= 4
DCX_LOCKWINDOWUPDATE= 0x400
DCX_EXCLUDERGN  = 64
DCX_INTERSECTRGN= 128
DCX_VALIDATE  = 0x200000

GCL_STYLE     = -26

SB_HORZ         =      0
SB_VERT         =      1
SB_CTL          =      2
SB_BOTH         =      3

SB_LINEUP         = 0
SB_LINELEFT       = 0
SB_LINEDOWN       = 1
SB_LINERIGHT      = 1
SB_PAGEUP         = 2
SB_PAGELEFT       = 2
SB_PAGEDOWN       = 3
SB_PAGERIGHT        = 3
SB_THUMBPOSITION    = 4
SB_THUMBTRACK       = 5
SB_TOP              = 6
SB_LEFT             = 6
SB_BOTTOM           = 7
SB_RIGHT            = 7
SB_ENDSCROLL        = 8

DT_TOP = 0x00000000
DT_LEFT = 0x00000000
DT_CENTER = 0x00000001
DT_RIGHT = 0x00000002
DT_VCENTER = 0x00000004
DT_BOTTOM = 0x00000008
DT_WORDBREAK = 0x00000010
DT_SINGLELINE = 0x00000020
DT_EXPANDTABS = 0x00000040
DT_TABSTOP = 0x00000080
DT_NOCLIP = 0x00000100
DT_EXTERNALLEADING = 0x00000200
DT_CALCRECT = 0x00000400
DT_NOPREFIX = 0x00000800
DT_INTERNAL = 0x00001000

DT_EDITCONTROL = 0x00002000
DT_PATH_ELLIPSIS = 0x00004000
DT_END_ELLIPSIS = 0x00008000
DT_MODIFYSTRING = 0x00010000
DT_RTLREADING = 0x00020000
DT_WORD_ELLIPSIS = 0x00040000
DT_NOFULLWIDTHCHARBREAK = 0x00080000
DT_HIDEPREFIX = 0x00100000
DT_PREFIXONLY = 0x00200000

MB_OK                    =   0x00000000
MB_OKCANCEL              =   0x00000001
MB_ABORTRETRYIGNORE      =   0x00000002
MB_YESNOCANCEL           =   0x00000003
MB_YESNO                 =   0x00000004
MB_RETRYCANCEL           =   0x00000005


MB_ICONASTERISK   = 64
MB_ICONEXCLAMATION  = 0x30
MB_ICONWARNING    = 0x30
MB_ICONERROR    = 16
MB_ICONHAND      = 16
MB_ICONQUESTION    = 32
MB_ICONINFORMATION  = 64
MB_ICONSTOP      = 16
MB_ICONMASK      = 240

IDOK          =  1
IDCANCEL      =  2
IDABORT       =  3
IDRETRY       =  4
IDIGNORE      =  5
IDYES         =  6
IDNO          =  7
IDCLOSE       =  8
IDHELP        =  9

COLOR_3DDKSHADOW = 21
COLOR_3DFACE  = 15
COLOR_3DHILIGHT = 20
COLOR_3DHIGHLIGHT= 20
COLOR_3DLIGHT= 22
COLOR_BTNHILIGHT= 20
COLOR_3DSHADOW= 16
COLOR_ACTIVEBORDER =10
COLOR_ACTIVECAPTION= 2
COLOR_APPWORKSPACE= 12
COLOR_BACKGROUND= 1
COLOR_DESKTOP= 1
COLOR_BTNFACE= 15
COLOR_BTNHIGHLIGHT= 20
COLOR_BTNSHADOW= 16
COLOR_BTNTEXT= 18
COLOR_CAPTIONTEXT= 9
COLOR_GRAYTEXT= 17
COLOR_HIGHLIGHT= 13
COLOR_HIGHLIGHTTEXT= 14
COLOR_INACTIVEBORDER= 11
COLOR_INACTIVECAPTION= 3
COLOR_INACTIVECAPTIONTEXT= 19
COLOR_INFOBK= 24
COLOR_INFOTEXT= 23
COLOR_MENU= 4
COLOR_MENUTEXT= 7
COLOR_SCROLLBAR= 0
COLOR_WINDOW= 5
COLOR_WINDOWFRAME= 6
COLOR_WINDOWTEXT= 8
CTLCOLOR_MSGBOX= 0
CTLCOLOR_EDIT= 1
CTLCOLOR_LISTBOX= 2
CTLCOLOR_BTN= 3
CTLCOLOR_DLG= 4
CTLCOLOR_SCROLLBAR= 5
CTLCOLOR_STATIC= 6
CTLCOLOR_MAX= 7

GW_HWNDFIRST = 0
GW_HWNDLAST = 1
GW_HWNDNEXT = 2
GW_HWNDPREV = 3
GW_OWNER = 4
GW_CHILD = 5
GW_ENABLEDPOPUP = 6

GMEM_FIXED         = 0x0000
GMEM_MOVEABLE      = 0x0002
GMEM_NOCOMPACT     = 0x0010
GMEM_NODISCARD     = 0x0020
GMEM_ZEROINIT      = 0x0040
GMEM_MODIFY        = 0x0080
GMEM_DISCARDABLE   = 0x0100
GMEM_NOT_BANKED    = 0x1000
GMEM_SHARE         = 0x2000
GMEM_DDESHARE      = 0x2000
GMEM_NOTIFY        = 0x4000
GMEM_LOWER         = GMEM_NOT_BANKED
GMEM_VALID_FLAGS   = 0x7F72
GMEM_INVALID_HANDLE= 0x8000

RT_DIALOG        = "5"

CF_TEXT = 1

PUSHBUTTON = 0x80
EDITTEXT = 0x81
LTEXT = 0x82
LISTBOX  = 0x83
SCROLLBAR = 0x84
COMBOXBOX = 0x85

ES_LEFT       = 0x0000
ES_CENTER     = 0x0001
ES_RIGHT      = 0x0002
ES_MULTILINE  = 0x0004
ES_UPPERCASE  = 0x0008
ES_LOWERCASE  = 0x0010
ES_PASSWORD   = 0x0020
ES_AUTOVSCROLL = 0x0040
ES_AUTOHSCROLL = 0x0080
ES_NOHIDESEL  = 0x0100
ES_OEMCONVERT = 0x0400
ES_READONLY   = 0x0800
ES_WANTRETURN = 0x1000
ES_NUMBER     = 0x2000

EN_SETFOCUS   = 0x0100
EN_KILLFOCUS  = 0x0200
EN_CHANGE     = 0x0300
EN_UPDATE     = 0x0400
EN_ERRSPACE   = 0x0500
EN_MAXTEXT    = 0x0501
EN_HSCROLL    = 0x0601
EN_VSCROLL    = 0x0602

EN_ALIGN_LTR_EC = 0x0700
EN_ALIGN_RTL_EC = 0x0701

EM_GETSEL = 0x00B0
EM_SETSEL = 0x00B1
EM_GETRECT = 0x00B2
EM_SETRECT = 0x00B3
EM_SETRECTNP = 0x00B4
EM_SCROLL = 0x00B5
EM_LINESCROLL = 0x00B6
EM_SCROLLCARET = 0x00B7
EM_GETMODIFY = 0x00B8
EM_SETMODIFY = 0x00B9
EM_GETLINECOUNT = 0x00BA
EM_LINEINDEX = 0x00BB
EM_LINELENGTH = 0x00C1
EM_REPLACESEL = 0x00C2
EM_GETLINE = 0x00C4
EM_LIMITTEXT = 0x00C5
EM_CANUNDO = 0x00C6
EM_UNDO = 0x00C7
EM_FMTLINES = 0x00C8
EM_LINEFROMCHAR = 0x00C9
EM_SETTABSTOPS = 0x00CB
EM_SETPASSWORDCHAR = 0x00CC
EM_EMPTYUNDOBUFFER = 0x00CD
EM_GETFIRSTVISIBLELINE = 0x00CE
EM_SETREADONLY = 0x00CF
EM_GETPASSWORDCHAR = 0x00D2


--#if(WINVER >= 0x0400)
EM_SETMARGINS = 0x00D3
EM_GETMARGINS = 0x00D4
EM_SETLIMITTEXT = EM_LIMITTEXT
EM_GETLIMITTEXT = 0x00D5
EM_POSFROMCHAR = 0x00D6
EM_CHARFROMPOS = 0x00D7
-- #endif /* WINVER >= 0x0400 */

CP_ACP = 0
DS_SETFONT = 0x40
DS_MODALFRAME = 0x80

SYNCHRONIZE  = 0x00100000
STANDARD_RIGHTS_REQUIRED = 0x000F0000
EVENT_ALL_ACCESS = bor(STANDARD_RIGHTS_REQUIRED,SYNCHRONIZE,0x3)
MAX_PATH = 260

function GET_XY_LPARAM(lParam)
  local x = LOWORD(lParam)
  if x > 32768 then
      x = x - 65536
  end
  local y = HIWORD(lParam)
  if y > 32768 then
      y = y - 65536
  end
  return x, y
end

function GET_POINT_LPARAM(lParam)
  local x, y = GET_XY_LPARAM(lParam)
  return POINT(x, y)
end

FVIRTKEY  = 0x01
FNOINVERT = 0x02
FSHIFT    = 0x04
FCONTROL  = 0x08
FALT      = 0x10

---------------------------------------------------------------------
-- macros

function LOWORD(dword)
    return band(dword, 0x0000ffff)
end

function HIWORD(dword)
    return rshift(dword, 16)
end

function MAKELONG(w1, w2)
    return bor(w1, lshift(w2,  16))
end

MAKELPARAM = MAKELONG

---------------------------------------------------------------------
-- GDI constants

TRANSPARENT = 1
OPAQUE = 2
BKMODE_LAST = 2

WHITE_BRUSH     = 0
LTGRAY_BRUSH  = 1
GRAY_BRUSH  = 2
DKGRAY_BRUSH  = 3
BLACK_BRUSH  = 4
NULL_BRUSH  = 5
HOLLOW_BRUSH = NULL_BRUSH
WHITE_PEN  = 6
BLACK_PEN  = 7
NULL_PEN  = 8
OEM_FIXED_FONT  = 10
ANSI_FIXED_FONT  = 11
ANSI_VAR_FONT  = 12
SYSTEM_FONT  = 13
DEVICE_DEFAULT_FONT  = 14
DEFAULT_PALETTE  = 15
SYSTEM_FIXED_FONT  = 16
DEFAULT_GUI_FONT  =  17

PS_SOLID = 0
PS_DASH = 1
PS_DOT = 2
PS_DASHDOT = 3
PS_DASHDOTDOT = 4
PS_NULL = 5
PS_INSIDEFRAME = 6
PS_USERSTYLE = 7
PS_ALTERNATE = 8
PS_STYLE_MASK = 0x0000000F

PS_ENDCAP_ROUND = 0x00000000
PS_ENDCAP_SQUARE = 0x00000100
PS_ENDCAP_FLAT = 0x00000200
PS_ENDCAP_MASK = 0x00000F00

PS_JOIN_ROUND = 0x00000000
PS_JOIN_BEVEL = 0x00001000
PS_JOIN_MITER = 0x00002000
PS_JOIN_MASK = 0x0000F000

PS_COSMETIC = 0x00000000
PS_GEOMETRIC = 0x00010000
PS_TYPE_MASK = 0x000F0000

---------------------------------------------------------------------
-- ListView constants

LVS_ICON = 0x0000
LVS_REPORT = 0x0001
LVS_SMALLICON = 0x0002
LVS_LIST = 0x0003
LVS_TYPEMASK = 0x0003
LVS_SINGLESEL = 0x0004
LVS_SHOWSELALWAYS = 0x0008
LVS_SORTASCENDING = 0x0010
LVS_SORTDESCENDING = 0x0020
LVS_SHAREIMAGELISTS = 0x0040
LVS_NOLABELWRAP = 0x0080
LVS_AUTOARRANGE = 0x0100
LVS_EDITLABELS = 0x0200
LVS_OWNERDATA = 0x1000
LVS_NOSCROLL = 0x2000
LVS_TYPESTYLEMASK = 0xfc00
LVS_ALIGNTOP = 0x0000
LVS_ALIGNLEFT = 0x0800
LVS_ALIGNMASK = 0x0c00
LVS_OWNERDRAWFIXED = 0x0400
LVS_NOCOLUMNHEADER = 0x4000
LVS_NOSORTHEADER = 0x8000

LVS_EX_GRIDLINES = 0x00000001
LVS_EX_SUBITEMIMAGES = 0x00000002
LVS_EX_CHECKBOXES = 0x00000004
LVS_EX_TRACKSELECT = 0x00000008
LVS_EX_HEADERDRAGDROP = 0x00000010
LVS_EX_FULLROWSELECT = 0x00000020
LVS_EX_ONECLICKACTIVATE = 0x00000040
LVS_EX_TWOCLICKACTIVATE = 0x00000080
-- #if (_WIN32_IE >= 0x0400)
LVS_EX_FLATSB = 0x00000100
LVS_EX_REGIONAL = 0x00000200
LVS_EX_INFOTIP = 0x00000400
LVS_EX_UNDERLINEHOT = 0x00000800
LVS_EX_UNDERLINECOLD = 0x00001000
LVS_EX_MULTIWORKAREAS = 0x00002000
-- #endif
-- #if (_WIN32_IE >= 0x0500)
LVS_EX_LABELTIP = 0x00004000
LVS_EX_BORDERSELECT = 0x00008000
-- #endif
-- #if (_WIN32_WINNT >= 0x501)
LVS_EX_DOUBLEBUFFER = 0x00010000
LVS_EX_HIDELABELS = 0x00020000
LVS_EX_SINGLEROW = 0x00040000
LVS_EX_SNAPTOGRID = 0x00080000
LVS_EX_SIMPLESELECT = 0x00100000
-- #endif
-- #if (_WIN32_WINNT >= 0x0600)
LVS_EX_JUSTIFYCOLUMNS = 0x00200000
LVS_EX_TRANSPARENTBKGND = 0x00400000
LVS_EX_TRANSPARENTSHADOWTEXT = 0x00800000
LVS_EX_AUTOAUTOARRANGE = 0x01000000
LVS_EX_HEADERINALLVIEWS = 0x02000000
LVS_EX_AUTOCHECKSELECT = 0x08000000
LVS_EX_AUTOSIZECOLUMNS = 0x10000000
LVS_EX_COLUMNSNAPPOINTS = 0x40000000
LVS_EX_COLUMNOVERFLOW = 0x80000000
-- #endif /* _WIN32_WINNT >= 0x0600 */

LVIF_TEXT = 0x0001
LVIF_IMAGE = 0x0002
LVIF_PARAM = 0x0004
LVIF_STATE = 0x0008
LVIF_INDENT = 0x0010
LVIF_NORECOMPUTE = 0x0800
-- #if (_WIN32_WINNT >= 0x501)
LVIF_GROUPID = 0x0100
LVIF_COLUMNS = 0x0200
-- #endif
-- #if (_WIN32_WINNT >= 0x0600)
LVIF_COLFMT = 0x00010000
-- #endif

LVIS_FOCUSED = 0x0001
LVIS_SELECTED = 0x0002
LVIS_CUT = 0x0004
LVIS_DROPHILITED = 0x0008
LVIS_ACTIVATING = 0x0020

LVIS_OVERLAYMASK = 0x0F00
LVIS_STATEIMAGEMASK = 0xF000

LVNI_ALL = 0x0000
LVNI_FOCUSED = 0x0001
LVNI_SELECTED = 0x0002
LVNI_CUT = 0x0004
LVNI_DROPHILITED = 0x0008

LVNI_ABOVE = 0x0100
LVNI_BELOW = 0x0200
LVNI_TOLEFT = 0x0400
LVNI_TORIGHT = 0x0800

LVFI_PARAM = 0x0001
LVFI_STRING = 0x0002
LVFI_PARTIAL = 0x0008
LVFI_WRAP = 0x0020
LVFI_NEARESTXY = 0x0040

LVIR_BOUNDS = 0
LVIR_ICON = 1
LVIR_LABEL = 2
LVIR_SELECTBOUNDS = 3

LVHT_NOWHERE = 0x0001
LVHT_ONITEMICON = 0x0002
LVHT_ONITEMLABEL = 0x0004
LVHT_ONITEMSTATEICON = 0x0008
LVHT_ONITEM = bit.bor(LVHT_ONITEMICON,LVHT_ONITEMLABEL,LVHT_ONITEMSTATEICON)
LVHT_ABOVE = 0x0008
LVHT_BELOW = 0x0010
LVHT_TORIGHT = 0x0020
LVHT_TOLEFT = 0x0040

LVA_DEFAULT = 0x0000
LVA_ALIGNLEFT = 0x0001
LVA_ALIGNTOP = 0x0002
LVA_SNAPTOGRID = 0x0005

LVCF_FMT = 0x0001
LVCF_WIDTH = 0x0002
LVCF_TEXT = 0x0004
LVCF_SUBITEM = 0x0008
LVCF_IMAGE = 0x0010
LVCF_ORDER = 0x0020
-- #if (_WIN32_WINNT >= 0x0600)
LVCF_MINWIDTH = 0x0040
LVCF_DEFAULTWIDTH = 0x0080
LVCF_IDEALWIDTH = 0x0100
-- #endif /* _WIN32_WINNT >= 0x0600 */

LVCFMT_LEFT = 0x0000
LVCFMT_RIGHT = 0x0001
LVCFMT_CENTER = 0x0002
LVCFMT_JUSTIFYMASK = 0x0003

LVCFMT_IMAGE = 0x0800
LVCFMT_BITMAP_ON_RIGHT = 0x1000
LVCFMT_COL_HAS_IMAGES = 0x8000

-- #if (_WIN32_WINNT >= 0x0600)
LVCFMT_FIXED_WIDTH = 0x00100
LVCFMT_NO_DPI_SCALE = 0x40000
LVCFMT_FIXED_RATIO = 0x80000
LVCFMT_LINE_BREAK = 0x100000
LVCFMT_FILL = 0x200000
LVCFMT_WRAP = 0x400000
LVCFMT_NO_TITLE = 0x800000
LVCFMT_TILE_PLACEMENTMASK = (LVCFMT_LINE_BREAK + LVCFMT_FILL)
LVCFMT_SPLITBUTTON = 0x1000000
-- #endif /* _WIN32_WINNT >= 0x0600 */


LVSCW_AUTOSIZE  = -1
LVSCW_AUTOSIZE_USEHEADER = -2


LVSICF_NOINVALIDATEALL = 0x00000001
LVSICF_NOSCROLL = 0x00000002

-- #if (_WIN32_IE >= 0x0400)
LV_MAX_WORKAREAS = 16

LVBKIF_SOURCE_NONE = 0x00000000
LVBKIF_SOURCE_HBITMAP = 0x00000001
LVBKIF_SOURCE_URL = 0x00000002
LVBKIF_SOURCE_MASK = 0x00000003
LVBKIF_STYLE_NORMAL = 0x00000000
LVBKIF_STYLE_TILE = 0x00000010
LVBKIF_STYLE_MASK = 0x00000010
-- #endif
-- #if (_WIN32_WINNT >= 0x501)
LVBKIF_FLAG_TILEOFFSET = 0x00000100
LVBKIF_TYPE_WATERMARK = 0x10000000
-- #endif

-- #if (_WIN32_IE >= 0x400)
LVKF_ALT = 0x0001
LVKF_CONTROL = 0x0002
LVKF_SHIFT = 0x0004
-- #endif

-- #if (_WIN32_WINNT >= 0x501)
LV_VIEW_ICON = 0x0000
LV_VIEW_DETAILS = 0x0001
LV_VIEW_SMALLICON = 0x0002
LV_VIEW_LIST = 0x0003
LV_VIEW_TILE = 0x0004
LV_VIEW_MAX = 0x0004

LVGF_NONE = 0x00000000
LVGF_HEADER = 0x00000001
LVGF_FOOTER = 0x00000002
LVGF_STATE = 0x00000004
LVGF_ALIGN = 0x00000008
LVGF_GROUPID = 0x00000010

-- #if (_WIN32_WINNT >= 0x0600)
LVGF_SUBTITLE = 0x00000100
LVGF_TASK = 0x00000200
LVGF_DESCRIPTIONTOP = 0x00000400
LVGF_DESCRIPTIONBOTTOM = 0x00000800
LVGF_TITLEIMAGE = 0x00001000
LVGF_EXTENDEDIMAGE = 0x00002000
LVGF_ITEMS = 0x00004000
LVGF_SUBSET = 0x00008000
LVGF_SUBSETITEMS = 0x00010000
-- #endif /* _WIN32_WINNT >= 0x0600 */

LVGS_NORMAL = 0x00000000
LVGS_COLLAPSED = 0x00000001
LVGS_HIDDEN = 0x00000002

LVGA_HEADER_LEFT = 0x00000001
LVGA_HEADER_CENTER = 0x00000002
LVGA_HEADER_RIGHT = 0x00000004
LVGA_FOOTER_LEFT = 0x00000008
LVGA_FOOTER_CENTER = 0x00000010
LVGA_FOOTER_RIGHT = 0x00000020

LVGMF_NONE = 0x00000000
LVGMF_BORDERSIZE = 0x00000001
LVGMF_BORDERCOLOR = 0x00000002
LVGMF_TEXTCOLOR = 0x00000004

LVTVIF_AUTOSIZE = 0x00000000
LVTVIF_FIXEDWIDTH = 0x00000001
LVTVIF_FIXEDHEIGHT = 0x00000002
LVTVIF_FIXEDSIZE = 0x00000003
-- #if (_WIN32_WINNT >= 0x0600)
LVTVIF_EXTENDED = 0x00000004
-- #endif /* _WIN32_WINNT >= 0x0600 */

LVTVIM_TILESIZE = 0x00000001
LVTVIM_COLUMNS = 0x00000002
LVTVIM_LABELMARGIN = 0x00000004

LVIM_AFTER = 0x00000001
-- #endif

LVIF_DI_SETITEM = 0x1000

LVGIT_UNFOLDED = 0x0001


LVM_FIRST = 0x1000

LVM_GETBKCOLOR = (LVM_FIRST+0)
LVM_SETBKCOLOR = (LVM_FIRST+1)
LVM_GETIMAGELIST = (LVM_FIRST+2)
LVM_SETIMAGELIST = (LVM_FIRST+3)
LVM_GETITEMCOUNT = (LVM_FIRST+4)
LVM_GETITEMA = (LVM_FIRST+5)
LVM_GETITEMW = (LVM_FIRST+75)
LVM_SETITEMA = (LVM_FIRST+6)
LVM_SETITEMW = (LVM_FIRST+76)
LVM_INSERTITEMA = (LVM_FIRST+7)
LVM_INSERTITEMW = (LVM_FIRST+77)
LVM_DELETEITEM = (LVM_FIRST+8)
LVM_DELETEALLITEMS = (LVM_FIRST+9)
LVM_GETCALLBACKMASK = (LVM_FIRST+10)
LVM_SETCALLBACKMASK = (LVM_FIRST+11)
LVM_GETNEXTITEM = (LVM_FIRST+12)
LVM_FINDITEMA = (LVM_FIRST+13)
LVM_FINDITEMW = (LVM_FIRST+83)
LVM_GETITEMRECT = (LVM_FIRST+14)
LVM_SETITEMPOSITION = (LVM_FIRST+15)
LVM_GETITEMPOSITION = (LVM_FIRST+16)
LVM_GETSTRINGWIDTHA = (LVM_FIRST+17)
LVM_GETSTRINGWIDTHW = (LVM_FIRST+87)
LVM_HITTEST = (LVM_FIRST+18)
LVM_ENSUREVISIBLE = (LVM_FIRST+19)
LVM_SCROLL = (LVM_FIRST+20)
LVM_REDRAWITEMS = (LVM_FIRST+21)
LVM_ARRANGE = (LVM_FIRST+22)
LVM_EDITLABELA = (LVM_FIRST+23)
LVM_EDITLABELW = (LVM_FIRST+118)
LVM_GETEDITCONTROL = (LVM_FIRST+24)
LVM_GETCOLUMNA = (LVM_FIRST+25)
LVM_GETCOLUMNW = (LVM_FIRST+95)
LVM_SETCOLUMNA = (LVM_FIRST+26)
LVM_SETCOLUMNW = (LVM_FIRST+96)
LVM_INSERTCOLUMNA = (LVM_FIRST+27)
LVM_INSERTCOLUMNW = (LVM_FIRST+97)
LVM_DELETECOLUMN = (LVM_FIRST+28)
LVM_GETCOLUMNWIDTH = (LVM_FIRST+29)
LVM_SETCOLUMNWIDTH = (LVM_FIRST+30)
LVM_GETHEADER = (LVM_FIRST+31)
LVM_CREATEDRAGIMAGE = (LVM_FIRST+33)
LVM_GETVIEWRECT = (LVM_FIRST+34)
LVM_GETTEXTCOLOR = (LVM_FIRST+35)
LVM_SETTEXTCOLOR = (LVM_FIRST+36)
LVM_GETTEXTBKCOLOR = (LVM_FIRST+37)
LVM_SETTEXTBKCOLOR = (LVM_FIRST+38)
LVM_GETTOPINDEX = (LVM_FIRST+39)
LVM_GETCOUNTPERPAGE = (LVM_FIRST+40)
LVM_GETORIGIN = (LVM_FIRST+41)
LVM_UPDATE = (LVM_FIRST+42)
LVM_SETITEMSTATE = (LVM_FIRST+43)
LVM_GETITEMSTATE = (LVM_FIRST+44)
LVM_GETITEMTEXTA = (LVM_FIRST+45)
LVM_GETITEMTEXTW = (LVM_FIRST+115)
LVM_SETITEMTEXTA = (LVM_FIRST+46)
LVM_SETITEMTEXTW = (LVM_FIRST+116)
LVM_SETITEMCOUNT = (LVM_FIRST+47)
LVM_SORTITEMS = (LVM_FIRST+48)
LVM_SETITEMPOSITION32 = (LVM_FIRST+49)
LVM_GETSELECTEDCOUNT = (LVM_FIRST+50)
LVM_GETITEMSPACING = (LVM_FIRST+51)
LVM_GETISEARCHSTRINGA = (LVM_FIRST+52)
LVM_GETISEARCHSTRINGW = (LVM_FIRST+117)
LVM_SETICONSPACING = (LVM_FIRST+53)
LVM_SETEXTENDEDLISTVIEWSTYLE = (LVM_FIRST+54)
LVM_GETEXTENDEDLISTVIEWSTYLE = (LVM_FIRST+55)
LVM_GETSUBITEMRECT = (LVM_FIRST+56)
LVM_SUBITEMHITTEST = (LVM_FIRST+57)
LVM_SETCOLUMNORDERARRAY = (LVM_FIRST+58)
LVM_GETCOLUMNORDERARRAY = (LVM_FIRST+59)
LVM_SETHOTITEM = (LVM_FIRST+60)
LVM_GETHOTITEM = (LVM_FIRST+61)
LVM_SETHOTCURSOR = (LVM_FIRST+62)
LVM_GETHOTCURSOR = (LVM_FIRST+63)
LVM_APPROXIMATEVIEWRECT = (LVM_FIRST+64)
-- #if (_WIN32_IE >= 0x0400)
LVM_SETWORKAREAS = (LVM_FIRST+65)
LVM_GETWORKAREAS = (LVM_FIRST+70)
LVM_GETNUMBEROFWORKAREAS = (LVM_FIRST+73)
LVM_GETSELECTIONMARK = (LVM_FIRST+66)
LVM_SETSELECTIONMARK = (LVM_FIRST+67)
LVM_SETHOVERTIME = (LVM_FIRST+71)
LVM_GETHOVERTIME = (LVM_FIRST+72)
LVM_SETTOOLTIPS = (LVM_FIRST+74)
LVM_GETTOOLTIPS = (LVM_FIRST+78)
LVM_SORTITEMSEX = (LVM_FIRST+81)
LVM_SETBKIMAGEA = (LVM_FIRST+68)
LVM_SETBKIMAGEW = (LVM_FIRST+138)
LVM_GETBKIMAGEA = (LVM_FIRST+69)
LVM_GETBKIMAGEW = (LVM_FIRST+139)
-- #endif /* _WIN32_IE >= 0x0400 */
-- #if (_WIN32_WINNT >= 0x501)
LVM_SETSELECTEDCOLUMN = (LVM_FIRST+140)
LVM_SETTILEWIDTH = (LVM_FIRST+141)
LVM_SETVIEW = (LVM_FIRST+142)
LVM_GETVIEW = (LVM_FIRST+143)
LVM_INSERTGROUP = (LVM_FIRST+145)
LVM_SETGROUPINFO = (LVM_FIRST+147)
LVM_GETGROUPINFO = (LVM_FIRST+149)
LVM_REMOVEGROUP = (LVM_FIRST+150)
LVM_MOVEGROUP = (LVM_FIRST+151)
LVM_MOVEITEMTOGROUP = (LVM_FIRST+154)
LVM_SETGROUPMETRICS = (LVM_FIRST+155)
LVM_GETGROUPMETRICS = (LVM_FIRST+156)
LVM_ENABLEGROUPVIEW = (LVM_FIRST+157)
LVM_SORTGROUPS = (LVM_FIRST+158)
LVM_INSERTGROUPSORTED = (LVM_FIRST+159)
LVM_REMOVEALLGROUPS = (LVM_FIRST+160)
LVM_HASGROUP = (LVM_FIRST+161)
LVM_SETTILEVIEWINFO = (LVM_FIRST+162)
LVM_GETTILEVIEWINFO = (LVM_FIRST+163)
LVM_SETTILEINFO = (LVM_FIRST+164)
LVM_GETTILEINFO = (LVM_FIRST+165)
LVM_SETINSERTMARK = (LVM_FIRST+166)
LVM_GETINSERTMARK = (LVM_FIRST+167)
LVM_INSERTMARKHITTEST = (LVM_FIRST+168)
LVM_GETINSERTMARKRECT = (LVM_FIRST+169)
LVM_SETINSERTMARKCOLOR = (LVM_FIRST+170)
LVM_GETINSERTMARKCOLOR = (LVM_FIRST+171)
LVM_SETINFOTIP = (LVM_FIRST+173)
LVM_GETSELECTEDCOLUMN = (LVM_FIRST+174)
LVM_ISGROUPVIEWENABLED = (LVM_FIRST+175)
LVM_GETOUTLINECOLOR = (LVM_FIRST+176)
LVM_SETOUTLINECOLOR = (LVM_FIRST+177)
LVM_CANCELEDITLABEL = (LVM_FIRST+179)
LVM_MAPINDEXTOID = (LVM_FIRST+180)
LVM_MAPIDTOINDEX = (LVM_FIRST+181)
-- #endif /* _WIN32_WINNT >= 0x501 */
-- #if (_WIN32_WINNT >= 0x0600)
LVM_GETEMPTYTEXT = (LVM_FIRST + 204)
LVM_GETFOOTERRECT = (LVM_FIRST + 205)
LVM_GETFOOTERINFO = (LVM_FIRST + 206)
LVM_GETFOOTERITEMRECT = (LVM_FIRST + 207)
LVM_GETFOOTERITEM = (LVM_FIRST + 208)
LVM_GETITEMINDEXRECT = (LVM_FIRST + 209)
LVM_SETITEMINDEXSTATE = (LVM_FIRST + 210)
LVM_GETNEXTITEMINDEX = (LVM_FIRST + 211)
-- #endif /* _WIN32_WINNT >= 0x0600 */


LVN_FIRST = (0xffffffff - 100 + 1)


LVN_ITEMCHANGING = (LVN_FIRST-0)
LVN_ITEMCHANGED = (LVN_FIRST-1)
LVN_INSERTITEM = (LVN_FIRST-2)
LVN_DELETEITEM = (LVN_FIRST-3)
LVN_DELETEALLITEMS = (LVN_FIRST-4)
LVN_BEGINLABELEDITA = (LVN_FIRST-5)
LVN_BEGINLABELEDITW = (LVN_FIRST-75)
LVN_ENDLABELEDITA = (LVN_FIRST-6)
LVN_ENDLABELEDITW = (LVN_FIRST-76)
LVN_COLUMNCLICK = (LVN_FIRST-8)
LVN_BEGINDRAG = (LVN_FIRST-9)
LVN_BEGINRDRAG = (LVN_FIRST-11)
LVN_ODCACHEHINT = (LVN_FIRST-13)
LVN_ODFINDITEMA = (LVN_FIRST-52)
LVN_ODFINDITEMW = (LVN_FIRST-79)
LVN_ITEMACTIVATE = (LVN_FIRST-14)
LVN_ODSTATECHANGED = (LVN_FIRST-15)
-- #if (_WIN32_IE >= 0x0400)
LVN_HOTTRACK = (LVN_FIRST-21)
-- #endif /* _WIN32_IE >= 0x0400 */
LVN_GETDISPINFOA = (LVN_FIRST-50)
LVN_GETDISPINFOW = (LVN_FIRST-77)
LVN_SETDISPINFOA = (LVN_FIRST-51)
LVN_SETDISPINFOW = (LVN_FIRST-78)
LVN_KEYDOWN = (LVN_FIRST-55)
LVN_MARQUEEBEGIN = (LVN_FIRST-56)
LVN_GETINFOTIPA = (LVN_FIRST-57)
LVN_GETINFOTIPW = (LVN_FIRST-58)
-- #if (_WIN32_IE >= 0x0500)
LVN_INCREMENTALSEARCHA = (LVN_FIRST-62)
LVN_INCREMENTALSEARCHW = (LVN_FIRST-63)

-- LVNSCH_DEFAULT = -1
-- LVNSCH_ERROR   = -2
-- LVNSCH_IGNORE  = -3

-- #endif /* _WIN32_IE >= 0x0500 */
-- #if (_WIN32_WINNT >= 0x501)
LVN_BEGINSCROLL = (LVN_FIRST-80)
LVN_ENDSCROLL = (LVN_FIRST-81)
-- #endif /* _WIN32_WINNT >= 0x501 */
-- #if (_WIN32_WINNT >= 0x0600)
LVN_COLUMNDROPDOWN = (LVN_FIRST-64)
LVN_COLUMNOVERFLOWCLICK = (LVN_FIRST-66)
LVN_LINKCLICK = (LVN_FIRST-84)
LVN_GETEMPTYMARKUP = (LVN_FIRST-87)
EMF_CENTERED = 0x00000001
-- #endif /* _WIN32_WINNT >= 0x0600 */

---------------------------------------------------------------------
-- treeview constants

TVI_ROOT  = -0x10000
TVI_FIRST = -0x0FFFF
TVI_LAST  = -0x0FFFE
TVI_SORT  = -0x0FFFD

TVS_HASBUTTONS = 0x0001
TVS_HASLINES = 0x0002
TVS_LINESATROOT = 0x0004
TVS_EDITLABELS = 0x0008
TVS_DISABLEDRAGDROP = 0x0010
TVS_SHOWSELALWAYS = 0x0020
TVS_RTLREADING = 0x0040

TVS_NOTOOLTIPS = 0x0080
TVS_CHECKBOXES = 0x0100
TVS_TRACKSELECT = 0x0200
--#if (_WIN32_IE >= 0x0400)
TVS_SINGLEEXPAND = 0x0400
TVS_INFOTIP = 0x0800
TVS_FULLROWSELECT = 0x1000
TVS_NOSCROLL = 0x2000
TVS_NONEVENHEIGHT = 0x4000
--#endif /* _WIN32_IE >= 0x0400 */
--#if (_WIN32_IE >= 0x500)
TVS_NOHSCROLL = 0x8000
--#endif /* _WIN32_IE >= 0x500 */

--#if (_WIN32_WINNT >= 0x0600)
TVS_EX_MULTISELECT = 0x0002
TVS_EX_DOUBLEBUFFER = 0x0004
TVS_EX_NOINDENTSTATE = 0x0008
TVS_EX_RICHTOOLTIP = 0x0010
TVS_EX_AUTOHSCROLL = 0x0020
TVS_EX_FADEINOUTEXPANDOS = 0x0040
TVS_EX_PARTIALCHECKBOXES = 0x0080
TVS_EX_EXCLUSIONCHECKBOXES = 0x0100
TVS_EX_DIMMEDCHECKBOXES = 0x0200
TVS_EX_DRAWIMAGEASYNC = 0x0400
--#endif /* _WIN32_WINNT >= 0x0600 */

TVIF_TEXT = 0x0001
TVIF_IMAGE = 0x0002
TVIF_PARAM = 0x0004
TVIF_STATE = 0x0008
TVIF_HANDLE = 0x0010
TVIF_SELECTEDIMAGE = 0x0020
TVIF_CHILDREN = 0x0040
--#if (_WIN32_IE >= 0x0400)
TVIF_INTEGRAL = 0x0080
--#endif
--#if (_WIN32_IE >= 0x0600)
TVIF_STATEEX = 0x0100
TVIF_EXPANDEDIMAGE = 0x0200
--#endif
TVIS_SELECTED = 0x0002
TVIS_CUT = 0x0004
TVIS_DROPHILITED = 0x0008
TVIS_BOLD = 0x0010
TVIS_EXPANDED = 0x0020
TVIS_EXPANDEDONCE = 0x0040
TVIS_EXPANDPARTIAL = 0x0080
TVIS_OVERLAYMASK = 0x0F00
TVIS_STATEIMAGEMASK = 0xF000
TVIS_USERMASK = 0xF000

--#if (_WIN32_IE >= 0x0600)
TVIS_EX_FLAT = 0x0001
--#if (_WIN32_WINNT >= 0x0600)
TVIS_EX_DISABLED = 0x0002
--#endif /* _WIN32_WINNT >= 0x0600 */
TVIS_EX_ALL = 0x0002
--#endif /* _WIN32_IE >= 0x0600 */

TVE_COLLAPSE = 0x0001
TVE_EXPAND = 0x0002
TVE_TOGGLE = 0x0003
TVE_EXPANDPARTIAL = 0x4000
TVE_COLLAPSERESET = 0x8000

TVSIL_NORMAL = 0
TVSIL_STATE = 2

TVGN_ROOT = 0x0000
TVGN_NEXT = 0x0001
TVGN_PREVIOUS = 0x0002
TVGN_PARENT = 0x0003
TVGN_CHILD = 0x0004
TVGN_FIRSTVISIBLE = 0x0005
TVGN_NEXTVISIBLE = 0x0006
TVGN_PREVIOUSVISIBLE = 0x0007
TVGN_DROPHILITE = 0x0008
TVGN_CARET = 0x0009
--#if (_WIN32_IE >= 0x0400)
TVGN_LASTVISIBLE = 0x000A
--#endif
--#if (_WIN32_IE >= 0x0600)
TVGN_NEXTSELECTED = 0x000B
--#endif

--#if (_WIN32_WINNT >= 0x501)
TVSI_NOSINGLEEXPAND = 0x8000
--#endif

TV_FIRST = 0x1100

TVM_INSERTITEMA = (TV_FIRST+0)
TVM_INSERTITEMW = (TV_FIRST+50)
TVM_DELETEITEM = (TV_FIRST+1)
TVM_EXPAND = (TV_FIRST+2)
TVM_GETITEMRECT = (TV_FIRST+4)
TVM_GETCOUNT = (TV_FIRST+5)
TVM_GETINDENT = (TV_FIRST+6)
TVM_SETINDENT = (TV_FIRST+7)
TVM_GETIMAGELIST = (TV_FIRST+8)
TVM_SETIMAGELIST = (TV_FIRST+9)
TVM_GETNEXTITEM = (TV_FIRST+10)
TVM_SELECTITEM = (TV_FIRST+11)
TVM_GETITEMA = (TV_FIRST+12)
TVM_GETITEMW = (TV_FIRST+62)
TVM_SETITEMA = (TV_FIRST+13)
TVM_SETITEMW = (TV_FIRST+63)
TVM_EDITLABELA = (TV_FIRST+14)
TVM_EDITLABELW = (TV_FIRST+65)
TVM_GETEDITCONTROL = (TV_FIRST+15)
TVM_GETVISIBLECOUNT = (TV_FIRST+16)
TVM_HITTEST = (TV_FIRST+17)
TVM_CREATEDRAGIMAGE = (TV_FIRST+18)
TVM_SORTCHILDREN = (TV_FIRST+19)
TVM_ENSUREVISIBLE = (TV_FIRST+20)
TVM_SORTCHILDRENCB = (TV_FIRST+21)
TVM_ENDEDITLABELNOW = (TV_FIRST+22)
TVM_GETISEARCHSTRINGA = (TV_FIRST+23)
TVM_GETISEARCHSTRINGW = (TV_FIRST+64)
TVM_SETTOOLTIPS = (TV_FIRST+24)
TVM_GETTOOLTIPS = (TV_FIRST+25)
--#if (_WIN32_IE >= 0x0400)
TVM_SETINSERTMARK = (TV_FIRST+26)
-- TVM_SETUNICODEFORMAT = CCM_SETUNICODEFORMAT
-- TVM_GETUNICODEFORMAT = CCM_GETUNICODEFORMAT
TVM_SETITEMHEIGHT = (TV_FIRST+27)
TVM_GETITEMHEIGHT = (TV_FIRST+28)
TVM_SETBKCOLOR = (TV_FIRST+29)
TVM_SETTEXTCOLOR = (TV_FIRST+30)
TVM_GETBKCOLOR = (TV_FIRST+31)
TVM_GETTEXTCOLOR = (TV_FIRST+32)
TVM_SETSCROLLTIME = (TV_FIRST+33)
TVM_GETSCROLLTIME = (TV_FIRST+34)
TVM_SETINSERTMARKCOLOR = (TV_FIRST+37)
TVM_GETINSERTMARKCOLOR = (TV_FIRST+38)
--#endif /* _WIN32_IE >= 0x0400 */
--#if (_WIN32_IE >= 0x0500)
TVM_GETITEMSTATE = (TV_FIRST+39)
TVM_SETLINECOLOR = (TV_FIRST+40)
TVM_GETLINECOLOR = (TV_FIRST+41)
--#endif /* _WIN32_IE >= 0x0500 */
--#if (_WIN32_WINNT >= 0x0501)
TVM_MAPACCIDTOHTREEITEM = (TV_FIRST+42)
TVM_MAPHTREEITEMTOACCID = (TV_FIRST+43)
TVM_SETEXTENDEDSTYLE = (TV_FIRST+44)
TVM_GETEXTENDEDSTYLE = (TV_FIRST+45)
TVM_SETAUTOSCROLLINFO = (TV_FIRST+59)
--#endif /* _WIN32_WINNT >= 0x0501 */
--#if (_WIN32_WINNT >= 0x0600)
TVM_GETSELECTEDCOUNT = (TV_FIRST+70)
TVM_SHOWINFOTIP = (TV_FIRST+71)
TVM_GETITEMPARTRECT = (TV_FIRST+72)
--#endif /* _WIN32_WINNT >= 0x0600 */


TVN_FIRST = (0xffffffff - 400 + 1)


TVN_SELCHANGINGA = (TVN_FIRST-1)
TVN_SELCHANGINGW = (TVN_FIRST-50)
TVN_SELCHANGEDA = (TVN_FIRST-2)
TVN_SELCHANGEDW = (TVN_FIRST-51)
TVN_GETDISPINFOA = (TVN_FIRST-3)
TVN_GETDISPINFOW = (TVN_FIRST-52)
TVN_SETDISPINFOA = (TVN_FIRST-4)
TVN_SETDISPINFOW = (TVN_FIRST-53)
TVN_ITEMEXPANDINGA = (TVN_FIRST-5)
TVN_ITEMEXPANDINGW = (TVN_FIRST-54)
TVN_ITEMEXPANDEDA = (TVN_FIRST-6)
TVN_ITEMEXPANDEDW = (TVN_FIRST-55)
TVN_BEGINDRAGA = (TVN_FIRST-7)
TVN_BEGINDRAGW = (TVN_FIRST-56)
TVN_BEGINRDRAGA = (TVN_FIRST-8)
TVN_BEGINRDRAGW = (TVN_FIRST-57)
TVN_DELETEITEMA = (TVN_FIRST-9)
TVN_DELETEITEMW = (TVN_FIRST-58)
TVN_BEGINLABELEDITA = (TVN_FIRST-10)
TVN_BEGINLABELEDITW = (TVN_FIRST-59)
TVN_ENDLABELEDITA = (TVN_FIRST-11)
TVN_ENDLABELEDITW = (TVN_FIRST-60)
TVN_KEYDOWN = (TVN_FIRST-12)
--#if (_WIN32_IE >= 0x0400)
TVN_GETINFOTIPA = (TVN_FIRST-13)
TVN_GETINFOTIPW = (TVN_FIRST-14)
TVN_SINGLEEXPAND = (TVN_FIRST-15)
--#endif

---------------------------------------------------------------------
-- tab control constants

TCS_SCROLLOPPOSITE = 0x0001
TCS_BOTTOM = 0x0002
TCS_RIGHT = 0x0002
TCS_MULTISELECT = 0x0004
--#if (_WIN32_IE >= 0x0400)
TCS_FLATBUTTONS = 0x0008
--#endif
TCS_FORCEICONLEFT = 0x0010
TCS_FORCELABELLEFT = 0x0020
TCS_HOTTRACK = 0x0040
TCS_VERTICAL = 0x0080
TCS_TABS = 0x0000
TCS_BUTTONS = 0x0100
TCS_SINGLELINE = 0x0000
TCS_MULTILINE = 0x0200
TCS_RIGHTJUSTIFY = 0x0000
TCS_FIXEDWIDTH = 0x0400
TCS_RAGGEDRIGHT = 0x0800
TCS_FOCUSONBUTTONDOWN = 0x1000
TCS_OWNERDRAWFIXED = 0x2000
TCS_TOOLTIPS = 0x4000
TCS_FOCUSNEVER = 0x8000

--#if (_WIN32_IE >= 0x0400)
TCS_EX_FLATSEPARATORS = 0x00000001
TCS_EX_REGISTERDROP = 0x00000002
--#endif

TCIF_TEXT = 0x0001
TCIF_IMAGE = 0x0002
TCIF_RTLREADING = 0x0004
TCIF_PARAM = 0x0008
TCIF_STATE = 0x0010

TCIS_BUTTONPRESSED = 0x0001
--#if (_WIN32_IE >= 0x0400)
TCIS_HIGHLIGHTED = 0x0002
--#endif

TCHT_NOWHERE = 0x0001
TCHT_ONITEMICON = 0x0002
TCHT_ONITEMLABEL = 0x0004
TCHT_ONITEM = bor(TCHT_ONITEMICON, TCHT_ONITEMLABEL)


TCM_FIRST = 0x1300

TCM_GETIMAGELIST = (TCM_FIRST+2)
TCM_SETIMAGELIST = (TCM_FIRST+3)
TCM_GETITEMCOUNT = (TCM_FIRST+4)
TCM_GETITEMA = (TCM_FIRST+5)
TCM_GETITEMW = (TCM_FIRST+60)
TCM_SETITEMA = (TCM_FIRST+6)
TCM_SETITEMW = (TCM_FIRST+61)
TCM_INSERTITEMA = (TCM_FIRST+7)
TCM_INSERTITEMW = (TCM_FIRST+62)
TCM_DELETEITEM = (TCM_FIRST+8)
TCM_DELETEALLITEMS = (TCM_FIRST+9)
TCM_GETITEMRECT = (TCM_FIRST+10)
TCM_GETCURSEL = (TCM_FIRST+11)
TCM_SETCURSEL = (TCM_FIRST+12)
TCM_HITTEST = (TCM_FIRST+13)
TCM_SETITEMEXTRA = (TCM_FIRST+14)
TCM_ADJUSTRECT = (TCM_FIRST+40)
TCM_SETITEMSIZE = (TCM_FIRST+41)
TCM_REMOVEIMAGE = (TCM_FIRST+42)
TCM_SETPADDING = (TCM_FIRST+43)
TCM_GETROWCOUNT = (TCM_FIRST+44)
TCM_GETTOOLTIPS = (TCM_FIRST+45)
TCM_SETTOOLTIPS = (TCM_FIRST+46)
TCM_GETCURFOCUS = (TCM_FIRST+47)
TCM_SETCURFOCUS = (TCM_FIRST+48)
TCM_SETMINTABWIDTH = (TCM_FIRST+49)
TCM_DESELECTALL = (TCM_FIRST+50)
--#if (_WIN32_IE >= 0x0400)
TCM_HIGHLIGHTITEM = (TCM_FIRST+51)
TCM_SETEXTENDEDSTYLE = (TCM_FIRST+52)
TCM_GETEXTENDEDSTYLE = (TCM_FIRST+53)
-- TCM_SETUNICODEFORMAT = CCM_SETUNICODEFORMAT
-- TCM_GETUNICODEFORMAT = CCM_GETUNICODEFORMAT
--#endif

TCN_FIRST = (0xffffffff - 550 + 1)

TCN_KEYDOWN     = (TCN_FIRST-0)
TCN_SELCHANGE   = (TCN_FIRST-1)
TCN_SELCHANGING = (TCN_FIRST-2)
-- #if (_WIN32_IE >= 0x0400)
TCN_GETOBJECT   = (TCN_FIRST-3)
-- #endif
-- #if (_WIN32_IE >= 0x0500)
TCN_FOCUSCHANGE = (TCN_FIRST-4)
-- #endif

-- toolbar constants

TB_ENABLEBUTTON = (WM_USER+1)
TB_CHECKBUTTON = (WM_USER+2)
TB_PRESSBUTTON = (WM_USER+3)
TB_HIDEBUTTON = (WM_USER+4)
TB_INDETERMINATE = (WM_USER+5)
TB_HIGHLIGHTBUTTON = (WM_USER+6)
TB_ISBUTTONENABLED = (WM_USER+9)
TB_ISBUTTONCHECKED = (WM_USER+10)
TB_ISBUTTONPRESSED = (WM_USER+11)
TB_ISBUTTONHIDDEN = (WM_USER+12)
TB_ISBUTTONINDETERMINATE = (WM_USER+13)
TB_ISBUTTONHIGHLIGHTED = (WM_USER+14)
TB_SETSTATE = (WM_USER+17)
TB_GETSTATE = (WM_USER+18)
TB_ADDBITMAP = (WM_USER+19)

TB_ADDBUTTONSA = (WM_USER+20)
TB_INSERTBUTTONA = (WM_USER+21)
TB_ADDBUTTONS = (WM_USER+20)
TB_INSERTBUTTON = (WM_USER+21)

TB_DELETEBUTTON = (WM_USER+22)
TB_GETBUTTON = (WM_USER+23)
TB_BUTTONCOUNT = (WM_USER+24)
TB_COMMANDTOINDEX = (WM_USER+25)

TB_ADDSTRINGA = (WM_USER+28)
TB_ADDSTRINGW = (WM_USER+77)
TB_GETITEMRECT = (WM_USER+29)
TB_BUTTONSTRUCTSIZE = (WM_USER+30)
TB_SETBUTTONSIZE = (WM_USER+31)
TB_SETBITMAPSIZE = (WM_USER+32)
TB_AUTOSIZE = (WM_USER+33)
TB_GETTOOLTIPS = (WM_USER+35)
if (UNDER_CE) then
TB_SETTOOLTIPS = (WM_USER+36)
else
TB_SETTOOLTIPS = (WM_USER+81)
end
TB_SETPARENT = (WM_USER+37)
TB_SETROWS = (WM_USER+39)
TB_GETROWS = (WM_USER+40)
TB_SETCMDID = (WM_USER+42)
TB_CHANGEBITMAP = (WM_USER+43)
TB_GETBITMAP = (WM_USER+44)
TB_GETBUTTONTEXTA = (WM_USER+45)
TB_GETBUTTONTEXTW = (WM_USER+75)
TB_REPLACEBITMAP = (WM_USER+46)
TB_SETINDENT = (WM_USER+47)
TB_SETIMAGELIST = (WM_USER+48)
TB_GETIMAGELIST = (WM_USER+49)
TB_LOADIMAGES = (WM_USER+50)
TB_GETRECT = (WM_USER+51)
TB_SETDISABLEDIMAGELIST= (WM_USER+54)
TB_GETDISABLEDIMAGELIST= (WM_USER+55)
TB_SETSTYLE = (WM_USER+56)
TB_GETSTYLE = (WM_USER+57)
TB_GETBUTTONSIZE = (WM_USER+58)
TB_SETBUTTONWIDTH = (WM_USER+59)
TB_SETMAXTEXTROWS = (WM_USER+60)
TB_GETTEXTROWS = (WM_USER+61)
TB_GETBUTTONINFOW = (WM_USER+63)
TB_SETBUTTONINFOW = (WM_USER+64)
TB_GETBUTTONINFOA = (WM_USER+65)
TB_SETBUTTONINFOA = (WM_USER+66)
TB_INSERTBUTTONW = (WM_USER+67)
TB_ADDBUTTONSW = (WM_USER+68)
TB_HITTEST = (WM_USER+69)

-- mask flags
TBIF_IMAGE = 0x00000001
TBIF_TEXT = 0x00000002
TBIF_STATE = 0x00000004
TBIF_STYLE = 0x00000008
TBIF_LPARAM = 0x00000010
TBIF_COMMAND = 0x00000020
TBIF_SIZE = 0x00000040

---------------------------------------------------------------------
-- other constants
TRUE       = 1
FALSE      = 0
NULL       = 0

FORMAT_MESSAGE_IGNORE_INSERTS  = 0
FORMAT_MESSAGE_FROM_SYSTEM     = 0

if (UNDER_CE) then

  SHIDIM_FLAGS = 0x0001

  SHIDIF_DONEBUTTON = 0x0001
  SHIDIF_SIZEDLG = 0x0002
  SHIDIF_SIZEDLGFULLSCREEN = 0x0004
  SHIDIF_SIPDOWN = 0x0008
  SHIDIF_FULLSCREENNOMENUBAR = 0x0010
  SHIDIF_EMPTYMENU = 0x0020
  SHIDIF_WANTSCROLLBAR = 0x0040
  SHIDIF_CANCELBUTTON = 0x0080


  SHCMBF_EMPTYBAR = 0x0001
  SHCMBF_HIDDEN = 0x0002
  SHCMBF_HIDESIPBUTTON = 0x0004
  SHCMBF_COLORBK = 0x0008
  SHCMBF_HMENU = 0x0010

  SHCMBM_SETSUBMENU = (WM_USER+400)
  SHCMBM_GETSUBMENU = (WM_USER+401)
  SHCMBM_GETMENU    = (WM_USER+402)
  SHCMBM_OVERRIDEKEY = (WM_USER+403)
  SHCMBM_SETBKCOLOR = (WM_USER+406)

  SHMBOF_NODEFAULT = 0x00000001
  SHMBOF_NOTIFY = 0x00000002

  SHFS_SHOWTASKBAR = 0x0001
  SHFS_HIDETASKBAR = 0x0002
  SHFS_SHOWSIPBUTTON = 0x0004
  SHFS_HIDESIPBUTTON = 0x0008
  SHFS_SHOWSTARTICON = 0x0010
  SHFS_HIDESTARTICON = 0x0020

  SHDB_SHOW = 0x0001
  SHDB_HIDE = 0x0002
  SHDB_SHOWCANCEL = 0x0004

end



-- GDI macros
function RGB(r,g,b)
    return bor(bor(r, lshift(g, 8)), lshift(b, 16))
end


---------------------------------------------------------------------
-- user32 functions

function CreateWindowW(lpClassName,lpWindowName,dwStyle,x,y,nWidth,nHeight,hWndParent,hMenu,hInstance,lpParam)
  return winapi.CreateWindowExW(0,lpClassName,lpWindowName,dwStyle,x,y,nWidth,nHeight,hWndParent,hMenu,hInstance,lpParam)
end

---------------------------------------------------------------------
-- gdi functions

function MoveTo(hdc, x, y)
  return winapi.MoveToEx(hdc, x, y, 0)
end

function GetTextExtent(hdc, text)
  local size = winapi.SIZE:new()
  winapi.GetTextExtentExPointW(hdc, text, (#text/2) - 1, 0, nil, nil, size)
  return size.cx, size.cy
end

---------------------------------------------------------------------
-- OS version

function isMajorMinor(self, major, minor)
  return { self.dwMajorVersion, self.dwMinorVersion } == { major, minor }
end

---------------------------------------------------------------------
-- error handling

function winapi.Assert(result)
  if (0 == result) then
    local errcode = winapi.GetLastError()
    if (S_OK ~= errcode) then
      error("Windows Error : " .. errcode, 2)
    end
  end
end

function ValidHandle(value)
  winapi.Assert(value)
  return value
end

---------------------------------------------------------------------
-- RECT extension functions

function winapi.RECT:width()
  return self.right - self.left
end

function winapi.RECT:height()
  return self.bottom - self.top
end

function winapi.RECT:size()
  return { width  = self.right - self.left,
           height = self.bottom - self.top }
end

function winapi.RECT:shrink(other)
  self.left   = venster.utils.limit_up (self.left  + other.left  , self.right);
  self.top    = venster.utils.limit_up (self.top   + other.top   , self.bottom);
  self.right  = venster.utils.limit_low(self.right - other.right , self.left);
  self.bottom = venster.utils.limit_low(self.bottom- other.bottom, self.top);
end

-- function ContainsPoint(self, pt)
--    return PtInRect(byref(self), pt)
-- end

-- WindowsCE specific stuff
if (UNDER_CE) then
  SHCMBF_EMPTYBAR = 0x0001
  SHCMBF_HIDDEN = 0x0002

  SHCMBF_HIDESIPBUTTON = 0x0004
  SHCMBF_COLORBK = 0x0008
  SHCMBF_HMENU = 0x0010

  SPI_SETSIPINFO = 224
  SPI_GETSIPINFO = 225

  SHIDIM_FLAGS = 0x0001

  SHIDIF_DONEBUTTON = 0x0001
  SHIDIF_SIZEDLG = 0x0002
  SHIDIF_SIZEDLGFULLSCREEN = 0x0004
  SHIDIF_SIPDOWN = 0x0008
  SHIDIF_FULLSCREENNOMENUBAR = 0x0010
  SHIDIF_EMPTYMENU = 0x0020
  SHIDIF_WANTSCROLLBAR = 0x0040
  SHIDIF_CANCELBUTTON = 0x0080
end

---------------------------------------------------------------------
-- coredll

if (UNDER_CE) then

  MSGQUEUE_NOPRECOMMIT  = 0x00000001
  MSGQUEUE_ALLOW_BROKEN = 0x00000002

  MSGQUEUE_MSGALERT     = 0x00000001

  POWER_STATE_ON           = 0x00010000        -- on state
  POWER_STATE_OFF          = 0x00020000        -- no power, full off
  POWER_STATE_CRITICAL     = 0x00040000        -- critical off
  POWER_STATE_BOOT         = 0x00080000        -- boot state
  POWER_STATE_IDLE         = 0x00100000        -- idle state
  POWER_STATE_SUSPEND      = 0x00200000        -- suspend state
  POWER_STATE_UNATTENDED   = 0x00400000        -- Unattended state.
  POWER_STATE_RESET        = 0x00800000        -- reset state
  POWER_STATE_USERIDLE     = 0x01000000        -- user idle state
  POWER_STATE_BACKLIGHTON  = 0x02000000        -- device scree backlight on
  POWER_STATE_PASSWORD     = 0x10000000        -- This state is password protected.

  function POWER_STATE(f)
    return band(f, 0xFFFF0000)
  end

  -- Power Requirement Flags
  POWER_NAME              = 0x00000001        -- Default
  POWER_FORCE             = 0x00001000
  POWER_DUMPDW            = 0x00002000        -- Calling CaptureDumpFileOnDevice() before entering this state.

  -- Power Broadcast Types -- there are up to 32 of these
  PBT_TRANSITION          = 0x00000001        -- broadcast specifying system power state transition
  PBT_RESUME              = 0x00000002        -- broadcast notifying a resume, specifies previous state
  PBT_POWERSTATUSCHANGE   = 0x00000004        -- power supply switched to/from AC/DC
  PBT_POWERINFOCHANGE     = 0x00000008        -- some system power status field has changed
  PBT_SUSPENDKEYPRESSED   = 0x00000100        -- Suspend Key has been pressed.

  -- OEMS may define power notifications starting with this ID and
  -- going up by powers of 2.
  PBT_OEMBASE             = 0x00010000

  -- This bitmask indicates that an application would like to receive all
  -- types of power notifications.
  POWER_NOTIFY_ALL        = 0xFFFFFFFF


end

---------------------------------------------------------------------
-- commdlg

OFN_READONLY = 0x00000001
OFN_OVERWRITEPROMPT = 0x00000002
OFN_HIDEREADONLY = 0x00000004
OFN_NOCHANGEDIR = 0x00000008
OFN_SHOWHELP = 0x00000010
OFN_ENABLEHOOK = 0x00000020
OFN_ENABLETEMPLATE = 0x00000040
OFN_ENABLETEMPLATEHANDLE = 0x00000080
OFN_NOVALIDATE = 0x00000100
OFN_ALLOWMULTISELECT = 0x00000200
OFN_EXTENSIONDIFFERENT = 0x00000400
OFN_PATHMUSTEXIST = 0x00000800
OFN_FILEMUSTEXIST = 0x00001000
OFN_CREATEPROMPT = 0x00002000
OFN_SHAREAWARE = 0x00004000
OFN_NOREADONLYRETURN = 0x00008000
OFN_NOTESTFILECREATE = 0x00010000
OFN_NONETWORKBUTTON = 0x00020000
OFN_NOLONGNAMES = 0x00040000
-- #if(WINVER >= 0x0400)
OFN_EXPLORER = 0x00080000
OFN_NODEREFERENCELINKS = 0x00100000
OFN_LONGNAMES = 0x00200000
-- #endif /* WINVER >= 0x0400 */
OFN_PROJECT = 0x00400000
OFN_PROPERTY = 0x00800000
OFN_SHOW_ALL = 0x01000000

OFN_SHAREFALLTHROUGH = 2
OFN_SHARENOWARN = 1
OFN_SHAREWARN = 0

OFNFAVDATA_SYNCHRONIZE = 0x80000000
OFNFAVDATA_AVAILOFFLINE = 0x40000000

CDN_FIRST = (0xffffffff - 601 + 1)
CDN_LAST  = (0xffffffff - 699 + 1)

CDN_INITDONE = (CDN_FIRST-0x0000)
CDN_SELCHANGE = (CDN_FIRST-0x0001)
CDN_FOLDERCHANGE = (CDN_FIRST-0x0002)
CDN_SHAREVIOLATION = (CDN_FIRST-0x0003)
CDN_HELP = (CDN_FIRST-0x0004)
CDN_FILEOK = (CDN_FIRST-0x0005)
CDN_TYPECHANGE = (CDN_FIRST-0x0006)

CDM_FIRST = (WM_USER+100)
CDM_LAST = (WM_USER+200)

CDM_GETSPEC = (CDM_FIRST+0x0000)
CDM_GETFILEPATH = (CDM_FIRST+0x0001)
CDM_GETFOLDERPATH = (CDM_FIRST+0x0002)
CDM_GETFOLDERIDLIST = (CDM_FIRST+0x0003)
CDM_SETCONTROLTEXT = (CDM_FIRST+0x0004)
CDM_HIDECONTROL = (CDM_FIRST+0x0005)
CDM_SETDEFEXT = (CDM_FIRST+0x0006)

CC_RGBINIT = 0x00000001
CC_FULLOPEN = 0x00000002
CC_PREVENTFULLOPEN = 0x00000004
CC_ENABLEHOOK = 0x00000010
CC_ENABLETEMPLATE = 0x00000020
CC_ENABLETEMPLATEHANDLE = 0x00000040
-- #if(WINVER >= 0x0400)
CC_SOLIDCOLOR = 0x00000080
CC_ANYCOLOR = 0x00000100
-- #endif /* WINVER >= 0x0400 */

PD_SELECTALLPAGES = 0x00000001
PD_SELECTSELECTION = 0x00000002
PD_SELECTDRAFTMODE = 0x00000008
PD_SELECTA4 = 0x00000010
PD_SELECTLETTER = 0x00000020
PD_SELECTINFRARED = 0x00000040
PD_SELECTSERIAL = 0x00000080

PD_DISABLEPAPERSIZE = 0x00000100
PD_DISABLEPRINTRANGE = 0x00000200
PD_DISABLEMARGINS = 0x00000400
PD_DISABLEORIENTATION = 0x00000800
PD_RETURNDEFAULTDC = 0x00002000
PD_ENABLEPRINTHOOK = 0x00004000
PD_ENABLEPRINTTEMPLATE = 0x00008000
PD_ENABLEPRINTTEMPLATEHANDLE = 0x00010000
PD_TITLE = 0x00020000

PD_SELECTPORTRAIT = 0x00040000
PD_SELECTLANDSCAPE = 0x00080000
PD_MARGINS = 0x00100000
PD_INTHOUSANDTHSOFINCHES = 0x00200000
PD_INHUNDREDTHSOFMILLIMETERS = 0x00400000
PD_MINMARGINS = 0x00800000

-- CommDlgExtendedError results
CDERR_DIALOGFAILURE = 0xFFFF
CDERR_STRUCTSIZE = 0x0001
CDERR_INITIALIZATION = 0x0002
CDERR_NOTEMPLATE = 0x0003
CDERR_FINDRESFAILURE = 0x0004
CDERR_LOADRESFAILURE = 0x0005
CDERR_LOCKRESFAILURE = 0x0006
CDERR_MEMALLOCFAILURE = 0x0007
CDERR_NOHOOK = 0x0008
CDERR_REGISTRYFAILURE = 0x0009
CDERR_NOHINSTANCE = 0x000A

PDERR_NODEFAULTPRN = 0x1000
PDERR_PRINTERNOTFOUND = 0x1001
PDERR_NOPORTS = 0x1002
PDERR_NOPRINTERS = 0x1003
PDERR_CREATEDCFAILURE = 0x1004

---------------------------------------------------------------------
-- ConnMgr

if (UNDER_CE) then

IID_DestNetInternet  = DEFINE_GUID(0x436ef144,0xb4fb,0x4863,0xa0,0x41,0x8f,0x90,0x5a,0x62,0xc5,0x72)
IID_DestNetCorp      = DEFINE_GUID(0xa1182988,0x0d73,0x439e,0x87,0xad,0x2a,0x5b,0x36,0x9f,0x80,0x8b)
IID_DestNetWAP       = DEFINE_GUID(0x7022e968,0x5a97,0x4051,0xbc,0x1c,0xc5,0x78,0xe2,0xfb,0xa5,0xd9)
IID_DestNetSecureWAP = DEFINE_GUID(0xf28d1f74,0x72be,0x4394,0xa4,0xa7,0x4e,0x29,0x62,0x19,0x39,0x0c)

CONNMGR_PARAM_GUIDDESTNET     = 0x1
CONNMGR_PARAM_MAXCOST         = 0x2
CONNMGR_PARAM_MINRCVBW        = 0x4
CONNMGR_PARAM_MAXCONNLATENCY  = 0x8

CONNMGR_FLAG_PROXY_HTTP       = 0x1
CONNMGR_FLAG_PROXY_WAP        = 0x2
CONNMGR_FLAG_PROXY_SOCKS4     = 0x4
CONNMGR_FLAG_PROXY_SOCKS5     = 0x8

CONNMGR_FLAG_SUSPEND_AWARE    = 0x10
CONNMGR_FLAG_REGISTERED_HOME  = 0x20
CONNMGR_FLAG_NO_ERROR_MSGS    = 0x40

CONNMGR_PRIORITY_VOICE        = 0x20000
CONNMGR_PRIORITY_USERINTERACTIVE = 0x08000
CONNMGR_PRIORITY_USERBACKGROUND  = 0x02000
CONNMGR_PRIORITY_USERIDLE     = 0x0800
CONNMGR_PRIORITY_HIPRIBKGND   = 0x0200
CONNMGR_PRIORITY_IDLEBKGND    = 0x0080
CONNMGR_PRIORITY_EXTERNALINTERACTIVE = 0x0020
CONNMGR_PRIORITY_LOWBKGND     = 0x0008
CONNMGR_PRIORITY_CACHED       = 0x0002
CONNMGR_PRIORITY_ALWAYS_ON    = 0x0001

CONNMGR_STATUS_UNKNOWN                = 0x00
CONNMGR_STATUS_CONNECTED              = 0x10
CONNMGR_STATUS_SUSPENDED              = 0x11
CONNMGR_STATUS_DISCONNECTED           = 0x20
CONNMGR_STATUS_CONNECTIONFAILED       = 0x21
CONNMGR_STATUS_CONNECTIONCANCELED     = 0x22
CONNMGR_STATUS_CONNECTIONDISABLED     = 0x23
CONNMGR_STATUS_NOPATHTODESTINATION    = 0x24
CONNMGR_STATUS_WAITINGFORPATH         = 0x25
CONNMGR_STATUS_WAITINGFORPHONE        = 0x26
CONNMGR_STATUS_PHONEOFF               = 0x27
CONNMGR_STATUS_EXCLUSIVECONFLICT      = 0x28
CONNMGR_STATUS_NORESOURCES            = 0x29
CONNMGR_STATUS_CONNECTIONLINKFAILED   = 0x2A
CONNMGR_STATUS_AUTHENTICATIONFAILED   = 0x2B
CONNMGR_STATUS_WAITINGCONNECTION      = 0x40
CONNMGR_STATUS_WAITINGFORRESOURCE     = 0x41
CONNMGR_STATUS_WAITINGFORNETWORK      = 0x42
CONNMGR_STATUS_WAITINGDISCONNECTION   = 0x80
CONNMGR_STATUS_WAITINGCONNECTIONABORT = 0x81

CONNMGR_MAX_DESC              = 128

end

return winapi
